@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005, 
-   2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2007, 2008, 2009, 2010
+   Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
    MA 02110-1301 USA */
 
 #include "comsat.h"
+#define MU_CFG_COMPATIBILITY /* This source uses deprecated cfg interfaces */
+#include "mailutils/libcfg.h"
 #include "mailutils/libargp.h"
 
 #ifndef PATH_DEV
@@ -54,7 +56,8 @@ typedef struct utmp UTMP;
 #define MAX_TTY_SIZE (sizeof (PATH_TTY_PFX) + sizeof (((UTMP*)0)->ut_line))
 
 const char *program_version = "comsatd (" PACKAGE_STRING ")";
-static char doc[] = "GNU comsatd";
+static char doc[] = N_("GNU comsatd -- the Comsat daemon.");
+static char args_doc[] = N_("\n--test MBOX-URL MSG-QID");
 
 #define OPT_FOREGROUND 256
 
@@ -62,12 +65,12 @@ static struct argp_option options[] =
 {
   { "config", 'c', N_("FILE"), OPTION_HIDDEN, "", 0 },
   { "convert-config", 'C', N_("FILE"), 0,
-    N_("Convert the configuration FILE to new format."), 0 },
-  { "test", 't', NULL, 0, N_("Run in test mode"), 0 },
-  { "foreground", OPT_FOREGROUND, 0, 0, N_("Remain in foreground."), 0},
-  { "inetd",  'i', 0, 0, N_("Run in inetd mode"), 0 },
+    N_("convert the configuration FILE to new format"), 0 },
+  { "test", 't', NULL, 0, N_("run in test mode"), 0 },
+  { "foreground", OPT_FOREGROUND, 0, 0, N_("remain in foreground"), 0},
+  { "inetd",  'i', 0, 0, N_("run in inetd mode"), 0 },
   { "daemon", 'd', N_("NUMBER"), OPTION_ARG_OPTIONAL,
-    N_("Runs in daemon mode with a maximum of NUMBER children"), 0 },
+    N_("runs in daemon mode with a maximum of NUMBER children"), 0 },
   { NULL, 0, NULL, 0, NULL, 0 }
 };
 
@@ -77,7 +80,7 @@ static error_t comsatd_parse_opt (int key, char *arg,
 static struct argp argp = {
   options,
   comsatd_parse_opt,
-  NULL, 
+  args_doc, 
   doc,
   NULL,
   NULL, NULL
@@ -120,7 +123,7 @@ int test_mode;
 
 struct mu_cfg_param comsat_cfg_param[] = {
   { "allow-biffrc", mu_cfg_bool, &allow_biffrc, 0, NULL,
-    N_("Read .biffrc file from the user home directory") },
+    N_("Read .biffrc file from the user home directory.") },
   { "require-tty", mu_cfg_bool, &require_tty, 0, NULL,
     N_("Notify only if the user is logged on one of the ttys.") },
   { "max-lines", mu_cfg_int, &maxlines, 0, NULL,
@@ -144,7 +147,7 @@ struct mu_cfg_param comsat_cfg_param[] = {
 static error_t
 comsatd_parse_opt (int key, char *arg, struct argp_state *state)
 {
-  static struct mu_argp_node_list lst;
+  static mu_list_t lst;
 
   switch (key)
     {
@@ -176,17 +179,17 @@ _("The old configuration file format and the --config command\n"
       exit (0);
 
     case 'd':
-      mu_argp_node_list_new (&lst, "mode", "daemon");
+      mu_argp_node_list_new (lst, "mode", "daemon");
       if (arg)
-	mu_argp_node_list_new (&lst, "max-children", arg);
+	mu_argp_node_list_new (lst, "max-children", arg);
       break;
 
     case 'i':
-      mu_argp_node_list_new (&lst, "mode", "inetd");
+      mu_argp_node_list_new (lst, "mode", "inetd");
       break;
 
     case OPT_FOREGROUND:
-      mu_argp_node_list_new (&lst, "foreground", "yes");
+      mu_argp_node_list_new (lst, "foreground", "yes");
       break;
 
     case 't':
@@ -198,7 +201,7 @@ _("The old configuration file format and the --config command\n"
       break;
       
     case ARGP_KEY_FINI:
-      mu_argp_node_list_finish (&lst, NULL, NULL);
+      mu_argp_node_list_finish (lst, NULL, NULL);
       break;
 
     default:
@@ -247,7 +250,7 @@ comsat_process (char *buffer, size_t rdlen)
   p = strchr (buffer, '@');
   if (!p)
     {
-      mu_diag_output (MU_DIAG_ERROR, _("Malformed input: %s"), buffer);
+      mu_diag_output (MU_DIAG_ERROR, _("malformed input: %s"), buffer);
       return;
     }
   *p++ = 0;
@@ -275,7 +278,7 @@ int
 comsat_main (int fd)
 {
   int rdlen;
-  int len;
+  socklen_t len;
   struct sockaddr fromaddr;
   char buffer[216]; /*FIXME: Arbitrary size */
 
@@ -321,8 +324,8 @@ comsat_prefork (int fd, void *data, struct sockaddr *s, int size)
 
 	  delay = overflow_delay_time << (overflow_count + 1);
 	  mu_diag_output (MU_DIAG_NOTICE,
-			 ngettext ("Too many requests: pausing for %u second",
-				   "Too many requests: pausing for %u seconds",
+			 ngettext ("too many requests: pausing for %u second",
+				   "too many requests: pausing for %u seconds",
 				    delay),
 			  delay);
 	  /* FIXME: drain the socket? */
@@ -365,8 +368,8 @@ comsat_connection (int fd, struct sockaddr *sa, int salen,
     {
       char *p = mu_sockaddr_to_astr (sa, salen);
       mu_diag_output (MU_DIAG_INFO,
-		      ngettext ("Received %d byte from %s",
-				"Received %d bytes from %s", rdlen),
+		      ngettext ("received %d byte from %s",
+				"received %d bytes from %s", rdlen),
 		      rdlen, p);
       mu_diag_output (MU_DIAG_INFO, "string: %s", buffer);
       free (p);
@@ -377,7 +380,7 @@ comsat_connection (int fd, struct sockaddr *sa, int salen,
       int rc = mu_udp_server_set_bufsize (srv, rdlen + 1);
       if (rc)
 	{
-	  mu_error (_("Cannot resize buffer: %s"), mu_strerror (rc));
+	  mu_error (_("cannot resize buffer: %s"), mu_strerror (rc));
 	  return 0;
 	}
     }
@@ -418,7 +421,7 @@ notify_user (const char *user, const char *device, const char *path,
     return;
   if ((fp = fopen (device, "w")) == NULL)
     {
-      mu_error (_("Cannot open device %s: %m"), device);
+      mu_error (_("cannot open device %s: %s"), device, mu_strerror (errno));
       return;
     }
 
@@ -434,7 +437,7 @@ notify_user (const char *user, const char *device, const char *path,
   if ((status = mu_mailbox_create (&mbox, path)) != 0
       || (status = mu_mailbox_open (mbox, MU_STREAM_READ|MU_STREAM_QACCESS)) != 0)
     {
-      mu_error (_("Cannot open mailbox %s: %s"),
+      mu_error (_("cannot open mailbox %s: %s"),
 	      path, mu_strerror (status));
       return;
     }
@@ -442,7 +445,7 @@ notify_user (const char *user, const char *device, const char *path,
   status = mu_mailbox_quick_get_message (mbox, qid, &msg);
   if (status)
     {
-      mu_error (_("Cannot get message (mailbox %s, qid %s): %s"),
+      mu_error (_("cannot get message (mailbox %s, qid %s): %s"),
 		path, qid, mu_strerror (status));
       return; /* FIXME: Notify the user, anyway */
     }
@@ -480,11 +483,12 @@ find_user (const char *name, char *tty)
 		   sizeof (ftty) - sizeof (PATH_DEV) - 2);
 	  ftty[sizeof (ftty) - 1] = 0;
 
-	  mu_normalize_path (ftty, "/");
+	  mu_normalize_path (ftty);
 	  if (strncmp (ftty, PATH_TTY_PFX, strlen (PATH_TTY_PFX)))
 	    {
 	      /* An attempt to break security... */
-	      mu_diag_output (MU_DIAG_ALERT, _("Bad line name in utmp record: %s"), ftty);
+	      mu_diag_output (MU_DIAG_ALERT,
+			      _("bad line name in utmp record: %s"), ftty);
 	      return NOT_HERE;
 	    }
 
@@ -492,7 +496,8 @@ find_user (const char *name, char *tty)
 	    {
 	      if (!S_ISCHR (statb.st_mode))
 		{
-		  mu_diag_output (MU_DIAG_ALERT, _("Not a character device: %s"), ftty);
+		  mu_diag_output (MU_DIAG_ALERT,
+				  _("not a character device: %s"), ftty);
 		  return NOT_HERE;
 		}
 
@@ -525,7 +530,7 @@ change_user (const char *user)
   pw = getpwnam (user);
   if (!pw)
     {
-      mu_diag_output (MU_DIAG_CRIT, _("No such user: %s"), user);
+      mu_diag_output (MU_DIAG_CRIT, _("no such user: %s"), user);
       return 1;
     }
 
@@ -546,7 +551,7 @@ mailbox_path (const char *user)
 
   if (!auth)
     {
-      mu_diag_output (MU_DIAG_ALERT, _("User nonexistent: %s"), user);
+      mu_diag_output (MU_DIAG_ALERT, _("user nonexistent: %s"), user);
       return NULL;
     }
 
@@ -592,7 +597,7 @@ main (int argc, char **argv)
   
       if (argc < 2 || argc > 2)
 	{
-	  mu_error (_("Mailbox URL and message QID are required in test mode"));
+	  mu_error (_("mailbox URL and message QID are required in test mode"));
 	  exit (EXIT_FAILURE);
 	}
 
@@ -605,7 +610,7 @@ main (int argc, char **argv)
 	      struct passwd *pw = getpwuid (getuid ());
 	      if (!pw)
 		{
-		  mu_error (_("Cannot determine user name"));
+		  mu_error (_("cannot determine user name"));
 		  exit (EXIT_FAILURE);
 		}
 	      user = pw->pw_name;
@@ -632,7 +637,7 @@ main (int argc, char **argv)
     {
       if (argv[0][0] != '/')
 	mu_diag_output (MU_DIAG_NOTICE,
-			_("Program name is not absolute; reloading will not "
+			_("program name is not absolute; reloading will not "
 			  "be possible"));
       else
 	{
@@ -650,7 +655,7 @@ main (int argc, char **argv)
       mu_m_server_destroy (&server);
       if (reload)
 	{
-	  mu_diag_output (MU_DIAG_NOTICE, _("Restarting"));
+	  mu_diag_output (MU_DIAG_NOTICE, _("restarting"));
 	  execvp (argv[0], argv);
 	}
     }
