@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
+   2009, 2010 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -62,10 +62,10 @@ static char *output_charset = NULL;
 const char *
 get_charset ()
 {
-  char *tmp;
-  
   if (!output_charset)
     {
+      char *tmp;
+      const char *str = NULL;
       char locale[32];
       
       memset (locale, 0, sizeof (locale));
@@ -86,16 +86,16 @@ get_charset ()
 	  
 	  lang = strtok_r (locale, "_", &sp);
 	  terr = strtok_r (NULL, ".", &sp);
-	  output_charset = strtok_r (NULL, "@", &sp);
+	  str = strtok_r (NULL, "@", &sp);
 
-	  if (output_charset)
-	    output_charset = xstrdup (output_charset);
-	  else
-	    output_charset = mu_charset_lookup (lang, terr);
+	  if (!str)
+	    str = mu_charset_lookup (lang, terr);
 	}
       
-      if (!output_charset)
-	output_charset = "ASCII";
+      if (!str)
+	str = "ASCII";
+
+      output_charset = xstrdup (str);
     }
   return output_charset;
 }
@@ -104,6 +104,20 @@ get_charset ()
 /* BIDI support (will be moved to lib when it's ready) */
 #ifdef HAVE_LIBFRIBIDI
 
+# ifdef HAVE_FRIBIDI_WCWIDTH
+#  define mu_fribidi_wcwidth fribidi_wcwidth
+# else
+#  if defined(HAVE_WCHAR_H) && defined(HAVE_WCWIDTH)
+#   include <wchar.h>
+#   define mu_fribidi_wcwidth(c) wcwidth((wchar_t)c)
+#  else
+#   undef HAVE_LIBFRIBIDI
+#  endif
+# endif
+#endif
+
+#ifdef HAVE_LIBFRIBIDI
+
 static int fb_charset_num = -1;
 FriBidiChar *logical;
 char *outstring;
@@ -170,7 +184,7 @@ puts_bidi (char *string)
 	      if (fb_charset_num != FRIBIDI_CHARSET_CAP_RTL)
 		{
 		  while (wid > 0 && idx < len)
-		    wid -= fribidi_wcwidth (visual[idx++]);
+		    wid -= mu_fribidi_wcwidth (visual[idx++]);
 		}
 	      else
 		{
@@ -371,7 +385,7 @@ rfc2047_decode_wrapper (const char *buf, size_t buflen)
   if (rc)
     {
       if (frm_debug)
-	mu_error (_("Cannot decode line `%s': %s"),
+	mu_error (_("cannot decode line `%s': %s"),
 		  buf, mu_strerror (rc));
       return strdup (buf);
     }
@@ -513,8 +527,8 @@ frm_abort (mu_mailbox_t *mbox)
   if ((status = mu_mailbox_close (*mbox)) != 0)
     {
       mu_url_t url;
-      
-      mu_error (_("Could not close mailbox `%s': %s"),
+      mu_mailbox_get_url (*mbox, &url);
+      mu_error (_("could not close mailbox `%s': %s"),
 		mu_url_to_string (url), mu_strerror (status));
       exit (3);
     }
@@ -542,10 +556,10 @@ frm_scan (char *mailbox_name, frm_select_t fun, size_t *total)
   if (status != 0)
     {
       if (mailbox_name)
-	mu_error (_("Could not create mailbox `%s': %s"),
+	mu_error (_("could not create mailbox `%s': %s"),
 		  mailbox_name,  mu_strerror (status));
       else
-	mu_error (_("Could not create default mailbox: %s"),
+	mu_error (_("could not create default mailbox: %s"),
 		  mu_strerror (status));
       exit (3);
     }
@@ -564,9 +578,10 @@ frm_scan (char *mailbox_name, frm_select_t fun, size_t *total)
     *total = 0;
   else if (status != 0)
     {
-      mu_error (_("Could not open mailbox `%s': %s"),
+      mu_error (_("could not open mailbox `%s': %s"),
 		mu_url_to_string (url), mu_strerror (status));
-      frm_abort (&mbox);
+      mu_mailbox_destroy (&mbox);
+      exit (3);
     }
   else
     {
@@ -584,7 +599,7 @@ frm_scan (char *mailbox_name, frm_select_t fun, size_t *total)
       status = mu_mailbox_scan (mbox, 1, total);
       if (status != 0)
 	{
-	  mu_error (_("Could not scan mailbox `%s': %s."),
+	  mu_error (_("could not scan mailbox `%s': %s."),
 		    mu_url_to_string (url), mu_strerror (status));
 	  frm_abort (&mbox);
 	}
@@ -594,7 +609,7 @@ frm_scan (char *mailbox_name, frm_select_t fun, size_t *total)
       
       if ((status = mu_mailbox_close (mbox)) != 0)
 	{
-	  mu_error (_("Could not close mailbox `%s': %s"),
+	  mu_error (_("could not close mailbox `%s': %s"),
 		    mu_url_to_string (url), mu_strerror (status));
 	  exit (3);
 	}
