@@ -1,10 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003,
-   2004, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008,
+   2009, 2010 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -18,6 +18,7 @@
    MA 02110-1301 USA */
 
 #include "guimb.h"
+#include "mailutils/libargp.h"
 
 char *program_file;
 char *program_expr;
@@ -42,7 +43,7 @@ append_arg (char *arg)
       g_argv = realloc (g_argv, g_size * sizeof (g_argv[0]));
       if (!g_argv)
 	{
-	  util_error (_("Not enough memory"));
+	  util_error (_("not enough memory"));
 	  exit (1);
 	}
     }
@@ -56,21 +57,21 @@ static struct argp_option options[] = {
    */
    N_("The following switches stop argument processing, and pass all\n"
    "remaining arguments as the value of (command-line):"), 1},
-  {"code", 'c', N_("EXPR"), 0, N_("Execute given scheme expression."), 1},
+  {"code", 'c', N_("EXPR"), 0, N_("execute given scheme expression"), 1},
   {"source", 's', N_("PROGFILE"), 0,
-   N_("Load Scheme source code from PROGFILE, and exit"), 1},
+   N_("load Scheme source code from PROGFILE and exit"), 1},
   {NULL, 0, NULL, 0,
    N_("The following options do not change the order of options parsing:"), 2},
-  {"expression", 'e', N_("EXPR"), 0, N_("Execute given scheme expression."), 2},
+  {"expression", 'e', N_("EXPR"), 0, N_("execute given scheme expression"), 2},
   {"file", 'f', N_("PROGFILE"), 0,
-   N_("Load Scheme source code from PROGFILE, and exit"), 2},
+   N_("load Scheme source code from PROGFILE and exit"), 2},
   {NULL, 0, NULL, 0, N_("Other options:"), 3},
-  {"debug", 'd', NULL, 0, N_("Start with debugging evaluator and backtraces."), 3},
+  {"debug", 'd', NULL, 0, N_("start with debugging evaluator and backtraces"), 3},
   {"guile-arg", 'g', N_("ARG"), 0,
-   N_("Append ARG to the command line passed to Guile"), 3},
-  {"mailbox", 'M', N_("NAME"), 0, N_("Set default mailbox name"), 3},
+   N_("append ARG to the command line passed to Guile"), 3},
+  {"mailbox", 'M', N_("NAME"), 0, N_("set default mailbox name"), 3},
   {"user", 'u', N_("NAME"), OPTION_ARG_OPTIONAL,
-   N_("Act as local MDA for user NAME"), 3},
+   N_("act as local MDA for user NAME"), 3},
   {0, 0, 0, 0}
 };
 
@@ -123,7 +124,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
 
 const char *program_version = "guimb (" PACKAGE_STRING ")";
 static char doc[] =
-N_("GNU guimb -- Process the contents of the specified mailboxes "
+N_("GNU guimb -- process contents of the specified mailboxes "
 "using a Scheme program or expression.");
 static char args_doc[] = N_("[mailbox...]");
 
@@ -138,25 +139,31 @@ static struct argp argp = {
 
 static const char *guimb_argp_capa[] = {
   "common",
+  "debug",
   "mailbox",
+  "locking",
   "license",
   NULL
 };
-    
+
+char *main_sym = "mailutils-main";
+
 int
 main (int argc, char *argv[])
 {
+  int rc;
   int c = argc;
   int index;
-  mu_guimb_param_t param;
-  struct guimb_data gd;
 
   /* Native Language Support */
-  mu_init_nls ();
+  MU_APP_INIT_NLS ();
+
+  /* Register the desired formats. */
+  mu_register_all_formats ();
 
-  append_arg ("");
   mu_argp_init (program_version, NULL);
-  mu_argp_parse (&argp, &argc, &argv, 0, guimb_argp_capa, &index, &c);
+  if (mu_app_init (&argp, guimb_argp_capa, NULL, argc, argv, 0, &index, &c))
+    exit (1);
 
   for (; c < argc; c++)
     append_arg (argv[c]);
@@ -164,17 +171,12 @@ main (int argc, char *argv[])
   if (!user_name)
     user_name = who_am_i ();
   
-  if (program_file)
-    g_argv[0] = program_file;
-  else if (!program_expr)
+  if (!program_file && !program_expr)
     {
       mu_error (_("At least one of -fecs must be used. Try guimb --help for more info."));
-      exit (0);
+      exit (1);
     }
     
-  /* Register the desired formats. */
-  mu_register_all_formats ();
-
   if (!argv[index])
     {
       if (default_mailbox)
@@ -197,27 +199,42 @@ main (int argc, char *argv[])
 	collect_append_file ("-");
     }
 
-  append_arg (NULL);
-  g_argc--;
-
   /* Finish creating input mailbox */
   collect_create_mailbox ();
 
-  gd.program_file = program_file;
-  gd.program_expr = program_expr;
+  mu_guile_init (debug_guile);
+  if (program_file)
+    mu_guile_load (program_file, g_argc, g_argv);
+  if (program_expr)
+    mu_guile_eval (program_expr);
+
+  rc = mu_guile_mailbox_apply (mbox, main_sym);
+  switch (rc)
+    {
+    case 0:
+      collect_output ();
+      break;
+
+    case MU_ERR_NOENT:
+      mu_error (_("%s not defined"), main_sym);
+      break;
+      
+    case EINVAL:
+      mu_error (_("%s is not a procedure object"), main_sym);
+      break;
+
+    case MU_ERR_FAILURE:
+      mu_error (_("execution of %s failed"), main_sym);
+      break;
+      
+    default:
+      mu_error (_("unrecognized error"));
+      break;
+    }
+
+  collect_drop_mailbox ();
   
-  param.debug_guile = debug_guile;
-  param.mbox = mbox;
-  param.user_name = user_name;
-  param.init = NULL;
-  param.catch_body = guimb_catch_body;
-  param.catch_handler = guimb_catch_handler;
-  param.next = NULL;
-  param.exit = guimb_exit;
-  param.data = &gd;
-  mu_process_mailbox (g_argc, g_argv, &param);
-  /*NOTREACHED*/
-  return 0;
+  return !!rc;
 }
 
 char *
