@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005, 2007, 2008, 2010 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,16 +20,23 @@
 #include "imap4d.h"
 
 /*
- *  
- */
+6.4.3.  EXPUNGE Command
+
+   Arguments:  none
+
+   Responses:  untagged responses: EXPUNGE
+
+   Result:     OK - expunge completed
+               NO - expunge failure: can't expunge (e.g. permission
+                    denied)
+               BAD - command unknown or arguments invalid
+*/
 
 int
-imap4d_expunge (struct imap4d_command *command, char *arg)
+imap4d_expunge (struct imap4d_command *command, imap4d_tokbuf_t tok)
 {
-  char *sp = NULL;
-
-  if (util_getword (arg, &sp))
-    return util_finish (command, RESP_NO, "Too many args");
+  if (imap4d_tokbuf_argc (tok) != 2)
+    return util_finish (command, RESP_BAD, "Invalid arguments");
 
   /* FIXME: check for errors.  */
   mu_mailbox_expunge (mbox);
