@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2007, 2008, 2010 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,13 +20,10 @@
 #include "imap4d.h"
 
 int
-imap4d_noop (struct imap4d_command *command, char *arg)
+imap4d_noop (struct imap4d_command *command, imap4d_tokbuf_t tok)
 {
-  char *sp = NULL;
-
-  if (util_getword (arg, &sp))
-    return util_finish (command, RESP_BAD, "Too many args");
-
+  if (imap4d_tokbuf_argc (tok) != 2)
+    return util_finish (command, RESP_BAD, "Invalid arguments");
   imap4d_sync ();
   return util_finish (command, RESP_OK, "Completed");
 }
