@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2007, 
-   2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2007, 2008, 2009, 2010 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -49,21 +49,25 @@ imap4d_master_signal (int signo)
 RETSIGTYPE
 imap4d_child_signal (int signo)
 {
+  imap4d_child_signal_setup (SIG_IGN);
   mu_diag_output (MU_DIAG_CRIT, _("got signal `%s'"), strsignal (signo));
-
   switch (signo)
     {
     case SIGTERM:
     case SIGHUP:
-      imap4d_bye (ERR_TERMINATE);
-
+      signo = ERR_TERMINATE;
+      break;
+      
     case SIGALRM:
-      imap4d_bye (ERR_TIMEOUT);
-
+      signo = ERR_TIMEOUT;
+      break;
+      
     case SIGPIPE:
-      imap4d_bye (ERR_NO_OFILE);
+      signo = ERR_NO_OFILE;
+      break;
       
     default:
-      imap4d_bye (ERR_SIGNAL);
+      signo = ERR_SIGNAL;
     }
+  imap4d_bye (signo);
 }
