@@ -1,11 +1,11 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2003, 2004, 
-   2005, 2006 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2003, 2004, 2005, 2006, 2007, 2009,
+   2010 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,7 +32,6 @@
 #include <assert.h>
 #include <time.h>
 
-#include <mu_umaxtostr.h>
 #include <mailutils/address.h>
 #include <mailutils/attribute.h>
 #include <mailutils/body.h>
@@ -46,6 +45,7 @@
 #include <mailutils/observer.h>
 #include <mailutils/property.h>
 #include <mailutils/stream.h>
+#include <mailutils/io.h>
 
 #include <imap0.h>
 #include <mailbox0.h>
@@ -100,8 +100,6 @@ static int  imap_attr_unset_flags (mu_attribute_t, int);
 
 /* mu_header_t API.  */
 static int  imap_header_read      (mu_header_t, char*, size_t, mu_off_t, size_t *);
-static int  imap_header_get_value (mu_header_t, const char*, char *, size_t, size_t *);
-static int  imap_header_get_fvalue (mu_header_t, const char*, char *, size_t, size_t *);
 
 /* mu_body_t API.  */
 static int  imap_body_read        (mu_stream_t, char *, size_t, mu_off_t, size_t *);
@@ -125,33 +123,30 @@ static int  is_same_folder        (mu_mailbox_t, mu_message_t);
 /* Initialize the concrete object mu_mailbox_t by overloading the function of the
    structure.  */
 int
-_mailbox_imap_init (mu_mailbox_t mailbox)
+_mailbox_imap_and_imaps_init (mu_mailbox_t mailbox, int imaps)
 {
+  int status;
   m_imap_t m_imap;
-  size_t name_len = 0;
-  mu_folder_t folder = NULL;
-
-  assert(mailbox);
-
-  folder = mailbox->folder;
 
+  if (!mailbox)
+    return MU_ERR_MBX_NULL;
+  if (mailbox->folder == NULL)
+    return EINVAL;
+  
   m_imap = mailbox->data = calloc (1, sizeof (*m_imap));
   if (m_imap == NULL)
     return ENOMEM;
 
   /* Retrieve the name of the mailbox from the URL.  */
-  mu_url_get_path (mailbox->url, NULL, 0, &name_len);
-  if (name_len == 0)
+  status = mu_url_aget_path (mailbox->url, &m_imap->name);
+  if (status == MU_ERR_NOENT)
     {
-      /* name "INBOX" is the default.  */
-      m_imap->name = calloc (6, sizeof (char));
-      strcpy (m_imap->name, "INBOX");
-    }
-  else
-    {
-      m_imap->name = calloc (name_len + 1, sizeof (char));
-      mu_url_get_path (mailbox->url, m_imap->name, name_len + 1, NULL);
+      m_imap->name = strdup ("INBOX");
+      if (!m_imap->name)
+        return ENOMEM;
     }
+  else if (status)
+    return status;
 
   /* Overload the functions.  */
   mailbox->_destroy = mailbox_imap_destroy;
@@ -173,8 +168,9 @@ _mailbox_imap_init (mu_mailbox_t mailbox)
   mailbox->_is_updated = imap_is_updated;
 
   /* Get the back pointer of the concrete folder. */
-  if (mailbox->folder)
-    m_imap->f_imap = mailbox->folder->data;
+  m_imap->f_imap = mailbox->folder->data;
+  m_imap->f_imap->imaps = imaps;
+  
   /* maibox back pointer.  */
   m_imap->mailbox = mailbox;
 
@@ -188,6 +184,19 @@ _mailbox_imap_init (mu_mailbox_t mailbox)
   return 0;
 }
 
+int
+_mailbox_imap_init (mu_mailbox_t mailbox)
+{
+  return _mailbox_imap_and_imaps_init (mailbox, 0);
+}
+
+int
+_mailbox_imaps_init (mu_mailbox_t mailbox)
+{
+  return _mailbox_imap_and_imaps_init (mailbox, 1);
+}
+
+
 /* Recursive call to free all the subparts of a message.  */
 static void
 free_subparts (msg_imap_t msg_imap)
@@ -300,21 +309,17 @@ mailbox_imap_open (mu_mailbox_t mailbox, int flags)
 	{
 	case IMAP_NO_STATE:
 	  {
-	    char *path;
-	    size_t len;
-	    mu_url_get_path (folder->url, NULL, 0, &len);
-	    if (len == 0)
+	    const char *path;
+	    status = mu_url_sget_path (folder->url, &path);
+	    if (status == MU_ERR_NOENT)
 	      return 0;
-	    path = calloc (len + 1, sizeof (*path));
-	    if (path == NULL)
-	      return ENOMEM;
-	    mu_url_get_path (folder->url, path, len + 1, NULL);
-	    status = imap_writeline (f_imap, "g%u CREATE %s\r\n",
-				     f_imap->seq, path);
-	    MAILBOX_DEBUG2 (folder, MU_DEBUG_PROT, "g%u CREATE %s\n",
-			    f_imap->seq, path);
+	    else if (status)
+	      return status;
+	    status = imap_writeline (f_imap, "g%lu CREATE %s\r\n",
+				     (unsigned long) f_imap->seq, path);
+	    MU_DEBUG2 (folder->debug, MU_DEBUG_PROT, "g%lu CREATE %s\n",
+		       (unsigned long) f_imap->seq, path);
 	    f_imap->seq++;
-	    free (path);
 	    if (status != 0)
 	      {
 		m_imap->state = f_imap->state = IMAP_NO_STATE;
@@ -388,10 +393,10 @@ mailbox_imap_close (mu_mailbox_t mailbox)
   switch (f_imap->state)
     {
     case IMAP_NO_STATE:
-      status = imap_writeline (f_imap, "g%s CLOSE\r\n",
-			       mu_umaxtostr (0, f_imap->seq++));
+      status = imap_writeline (f_imap, "g%lu CLOSE\r\n",
+			       (unsigned long) f_imap->seq++);
       CHECK_ERROR (f_imap, status);
-      MAILBOX_DEBUG0 (mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_CLOSE;
 
     case IMAP_CLOSE:
@@ -404,7 +409,7 @@ mailbox_imap_close (mu_mailbox_t mailbox)
 	size_t i;
 	status = imap_parse (f_imap);
 	CHECK_EAGAIN (f_imap, status);
-	MAILBOX_DEBUG0 (mailbox, MU_DEBUG_PROT, f_imap->buffer);
+	MU_DEBUG (mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
 
 	mu_monitor_wrlock (mailbox->monitor);
 	/* Destroy the imap messages and ressources associated to them.  */
@@ -546,8 +551,6 @@ imap_get_message0 (msg_imap_t msg_imap, mu_message_t *pmsg)
         return status;
       }
     mu_header_set_fill (header, imap_header_read, msg);
-    mu_header_set_get_value (header, imap_header_get_value, msg);
-    mu_header_set_get_fvalue (header, imap_header_get_fvalue, msg);
     mu_message_set_header (msg, header, msg_imap);
   }
 
@@ -679,12 +682,12 @@ imap_messages_count (mu_mailbox_t mailbox, size_t *pnum)
   switch (f_imap->state)
     {
     case IMAP_NO_STATE:
-      status = imap_writeline (f_imap, "g%s %s %s\r\n",
-			       mu_umaxtostr (0, f_imap->seq++), 
+      status = imap_writeline (f_imap, "g%lu %s %s\r\n",
+			       (unsigned long) f_imap->seq++, 
                                MBX_WRITABLE(mailbox) ? "SELECT" : "EXAMINE",
                                m_imap->name);
       CHECK_ERROR (f_imap, status);
-      MAILBOX_DEBUG0 (mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_SELECT;
 
     case IMAP_SELECT:
@@ -695,7 +698,7 @@ imap_messages_count (mu_mailbox_t mailbox, size_t *pnum)
     case IMAP_SELECT_ACK:
       status = imap_parse (f_imap);
       CHECK_EAGAIN (f_imap, status);
-      MAILBOX_DEBUG0 (mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
       break;
 
     default:
@@ -716,14 +719,14 @@ imap_scan (mu_mailbox_t mailbox, size_t msgno, size_t *pcount)
   return imap_scan0 (mailbox, msgno, pcount , 1);
 }
 
-/* Usually when this function is call it is because there is an oberver
-   attach an the client is try to build some sort of list/tree header
-   as the scanning progress.  But doing this for each message can be
-   time consuming and inefficient.  So we bundle all the request
-   into one and ask the server for everything "FETCH 1:*".  The good
-   side is that everything will be faster and we do not do lot of small
-   transcation but rather a big one.  The bad thing is that every thing
-   will be cache in the structure using a lot of memory.  */
+/* Normally this function is called when an observer is trying to build
+   some sort of list/tree header as the scanning progresses.  But doing
+   this for each message can be time consuming and inefficient.  So we
+   bundle all requests into one and ask the server for everything:
+   "FETCH 1:*".  The good side is that everything will be faster and we
+   do not do lots of small transcations, but rather a big one.  The bad
+   side is that everything will be cached in the structure using a lot of
+   memory.  */
 static int
 imap_scan0 (mu_mailbox_t mailbox, size_t msgno, size_t *pcount, int notif)
 {
@@ -748,11 +751,11 @@ imap_scan0 (mu_mailbox_t mailbox, size_t msgno, size_t *pcount, int notif)
     {
     case IMAP_NO_STATE:
       status = imap_writeline (f_imap,
-			       "g%s FETCH 1:* (FLAGS RFC822.SIZE BODY.PEEK[HEADER.FIELDS (%s)])\r\n",
-			       mu_umaxtostr (0, f_imap->seq++),
+			       "g%lu FETCH 1:* (FLAGS RFC822.SIZE BODY.PEEK[HEADER.FIELDS (%s)])\r\n",
+			       (unsigned long) f_imap->seq++,
 			       MU_IMAP_CACHE_HEADERS);
       CHECK_ERROR (f_imap, status);
-      MAILBOX_DEBUG0 (mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_SCAN;
 
     case IMAP_SCAN:
@@ -769,7 +772,7 @@ imap_scan0 (mu_mailbox_t mailbox, size_t msgno, size_t *pcount, int notif)
     case IMAP_SCAN_ACK:
       status = imap_parse (f_imap);
       CHECK_EAGAIN (f_imap, status);
-      MAILBOX_DEBUG0 (mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
       /* Clear the callback string structure.  */
       mu_stream_truncate (f_imap->string.stream, 0);
       f_imap->string.offset = 0;
@@ -795,12 +798,13 @@ imap_scan0 (mu_mailbox_t mailbox, size_t msgno, size_t *pcount, int notif)
 
   for (i = msgno; i <= count; i++)
     {
-      if (mu_observable_notify (mailbox->observable, MU_EVT_MESSAGE_ADD) != 0)
+      size_t tmp = i;
+      if (mu_observable_notify (mailbox->observable, MU_EVT_MESSAGE_ADD,
+                                &tmp) != 0)
 	break;
-      if (((i + 1) % 100) == 0)
-	{
-	  mu_observable_notify (mailbox->observable, MU_EVT_MAILBOX_PROGRESS);
-	}
+      if ((i + 1) % 100 == 0)
+	mu_observable_notify (mailbox->observable, MU_EVT_MAILBOX_PROGRESS, 
+                              NULL);
     }
   return 0;
 }
@@ -823,10 +827,10 @@ imap_is_updated (mu_mailbox_t mailbox)
   switch (f_imap->state)
     {
     case IMAP_NO_STATE:
-      status = imap_writeline (f_imap, "g%s NOOP\r\n",
-			       mu_umaxtostr (0, f_imap->seq++));
+      status = imap_writeline (f_imap, "g%lu NOOP\r\n",
+			       (unsigned long) f_imap->seq++);
       CHECK_ERROR (f_imap, status);
-      MAILBOX_DEBUG0 (mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_NOOP;
 
     case IMAP_NOOP:
@@ -837,7 +841,7 @@ imap_is_updated (mu_mailbox_t mailbox)
     case IMAP_NOOP_ACK:
       status = imap_parse (f_imap);
       CHECK_EAGAIN (f_imap, status);
-      MAILBOX_DEBUG0 (mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
       break;
 
     default:
@@ -878,12 +882,12 @@ imap_expunge (mu_mailbox_t mailbox)
 	    return 0;
 	  }
 	status = imap_writeline (f_imap,
-				 "g%s STORE %s +FLAGS.SILENT (\\Deleted)\r\n",
-				 mu_umaxtostr (0, f_imap->seq++),
+				 "g%lu STORE %s +FLAGS.SILENT (\\Deleted)\r\n",
+				 (unsigned long) f_imap->seq++,
 				 set);
 	free (set);
 	CHECK_ERROR (f_imap, status);
-	MAILBOX_DEBUG0 (m_imap->mailbox, MU_DEBUG_PROT, f_imap->buffer);
+	MU_DEBUG (m_imap->mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
 	f_imap->state = IMAP_STORE;
       }
 
@@ -896,13 +900,13 @@ imap_expunge (mu_mailbox_t mailbox)
     case IMAP_STORE_ACK:
       status = imap_parse (f_imap);
       CHECK_EAGAIN (f_imap, status);
-      MAILBOX_DEBUG0 (m_imap->mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (m_imap->mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_NO_STATE;
 
     case IMAP_EXPUNGE:
     case IMAP_EXPUNGE_ACK:
-      status = imap_writeline (f_imap, "g%s EXPUNGE\r\n",
-			       mu_umaxtostr (0, f_imap->seq++));
+      status = imap_writeline (f_imap, "g%lu EXPUNGE\r\n",
+			       (unsigned long) f_imap->seq++);
       CHECK_ERROR (f_imap, status);
       status = imap_send (f_imap);
       CHECK_EAGAIN (f_imap, status);
@@ -984,7 +988,7 @@ imap_append_message0 (mu_mailbox_t mailbox, mu_message_t msg)
     case IMAP_NO_STATE:
       {
 	size_t lines, size;
-	char *path;
+	const char *path;
 	char *abuf = malloc (1);
 	/* Get the desired flags attribute.  */
 	if (abuf == NULL)
@@ -1013,28 +1017,9 @@ imap_append_message0 (mu_mailbox_t mailbox, mu_message_t msg)
 	}
 
 	/* Get the mailbox filepath.  */
-	{
-	  size_t n = 0;
-	  mu_url_get_path (mailbox->url, NULL, 0, &n);
-	  if (n == 0)
-	    {
-	      if (!(path = strdup ("INBOX")))
-		{
-		  free (abuf);
-		  return ENOMEM;
-		}
-	    }
-	  else
-	    {
-	      path = calloc (n + 1, sizeof (*path));
-	      if (path == NULL)
-		{
-		  free (abuf);
-		  return ENOMEM;
-		}
-	      mu_url_get_path (mailbox->url, path, n + 1, NULL);
-	    }
-	}
+        status = mu_url_sget_path (mailbox->url, &path);
+        if (status == MU_ERR_NOENT)
+          path = "INBOX";
 
 	/* FIXME: we need to get the mu_envelope_date and use it.
 	   currently it is ignored.  */
@@ -1044,15 +1029,14 @@ imap_append_message0 (mu_mailbox_t mailbox, mu_message_t msg)
 	mu_message_size (msg, &size);
 	mu_message_lines (msg, &lines);
 	total = size + lines;
-	status = imap_writeline (f_imap, "g%s APPEND %s %s {%s}\r\n",
-				 mu_umaxtostr (0, f_imap->seq++),
+	status = imap_writeline (f_imap, "g%lu APPEND %s %s {%lu}\r\n",
+				 (unsigned long) f_imap->seq++,
 				 path,
 				 abuf,
-				 mu_umaxtostr (1, size + lines));
+				 (unsigned long) (size + lines));
 	free (abuf);
-	free (path);
 	CHECK_ERROR (f_imap, status);
-	MAILBOX_DEBUG0 (mailbox, MU_DEBUG_PROT, f_imap->buffer);
+	MU_DEBUG (mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
 	f_imap->state = IMAP_APPEND;
       }
 
@@ -1064,7 +1048,7 @@ imap_append_message0 (mu_mailbox_t mailbox, mu_message_t msg)
     case IMAP_APPEND_CONT:
       status = imap_parse (f_imap);
       CHECK_EAGAIN (f_imap, status);
-      MAILBOX_DEBUG0 (mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
       /* If we did not receive the continuation token, it is an error
          bail out.  */
       if (f_imap->buffer[0] != '+')
@@ -1097,15 +1081,15 @@ imap_append_message0 (mu_mailbox_t mailbox, mu_message_t msg)
 	  }
 	f_imap->state = IMAP_APPEND_ACK;
       }
-      /* !@#%$ UW-IMAP server hack: insists on the last line.  */
-      imap_writeline (f_imap, "\n");
+      /* !@#%$ UW-IMAP and Gimap server hack: both insist on the last line.  */
+      imap_writeline (f_imap, "\r\n");
       status = imap_send (f_imap);
       CHECK_EAGAIN (f_imap, status);
 
     case IMAP_APPEND_ACK:
       status = imap_parse (f_imap);
       CHECK_EAGAIN (f_imap, status);
-      MAILBOX_DEBUG0 (m_imap->mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (m_imap->mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
 
     default:
       /* mu_error ("imap_append: unknown state\n"); */
@@ -1136,23 +1120,16 @@ imap_copy_message (mu_mailbox_t mailbox, mu_message_t msg)
     {
     case IMAP_NO_STATE:
       {
-	char *path;
-	size_t n = 0;
+	const char *path;
 	/* Check for a valid mailbox name.  */
-	mu_url_get_path (mailbox->url, NULL, 0, &n);
-	if (n == 0)
-	  return EINVAL;
-	path = calloc (n + 1, sizeof (*path));
-	if (path == NULL)
-	  return ENOMEM;
-	mu_url_get_path (mailbox->url, path, n + 1, NULL);
-	status = imap_writeline (f_imap, "g%s COPY %s %s\r\n",
-				 mu_umaxtostr (0, f_imap->seq++),
-				 mu_umaxtostr (1, msg_imap->num),
-				 path);
-	free (path);
+	status = mu_url_sget_path (mailbox->url, &path);
+	if (status == 0)
+  	  status = imap_writeline (f_imap, "g%lu COPY %lu %s\r\n",
+	 			   (unsigned long) f_imap->seq++,
+				   (unsigned long) msg_imap->num,
+				   path);
 	CHECK_ERROR (f_imap, status);
-	MAILBOX_DEBUG0 (mailbox, MU_DEBUG_PROT, f_imap->buffer);
+	MU_DEBUG (mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
 	f_imap->state = IMAP_COPY;
       }
 
@@ -1164,7 +1141,7 @@ imap_copy_message (mu_mailbox_t mailbox, mu_message_t msg)
     case IMAP_COPY_ACK:
       status = imap_parse (f_imap);
       CHECK_EAGAIN (f_imap, status);
-      MAILBOX_DEBUG0 (mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
 
     default:
       break;
@@ -1218,17 +1195,17 @@ imap_message_read (mu_stream_t stream, char *buffer, size_t buflen,
       /* We have strip the \r, but the offset on the imap server is with that
 	 octet(CFLF) so add it in the offset, it's the number of lines.  */
       status = imap_writeline (f_imap,
-			       "g%s FETCH %s BODY.PEEK[%s]<%s.%s>\r\n",
-			       mu_umaxtostr (0, f_imap->seq++),
-			       mu_umaxtostr (1, msg_imap->num),
+			       "g%lu FETCH %lu BODY.PEEK[%s]<%lu.%lu>\r\n",
+			       (unsigned long) f_imap->seq++,
+			       (unsigned long) msg_imap->num,
 			       (section) ? section : "",
-			       mu_umaxtostr (2, offset +
-					     msg_imap->mu_message_lines),
-			       mu_umaxtostr (3, buflen));
+			       (unsigned long) (offset +
+						msg_imap->mu_message_lines),
+			       (unsigned long) buflen);
       if (section)
 	free (section);
       CHECK_ERROR (f_imap, status);
-      MAILBOX_DEBUG0 (m_imap->mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (m_imap->mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_FETCH;
     }
   status = fetch_operation (f_imap, msg_imap, buffer, buflen, plen);
@@ -1306,11 +1283,11 @@ imap_message_size (mu_message_t msg, size_t *psize)
 	     that octet so add it in the offset, since it's the number of
 	     lines.  */
 	  status = imap_writeline (f_imap,
-				   "g%s FETCH %s RFC822.SIZE\r\n",
-				   mu_umaxtostr (0, f_imap->seq++),
-				   mu_umaxtostr (1, msg_imap->num));
+				   "g%lu FETCH %lu RFC822.SIZE\r\n",
+				   (unsigned long) f_imap->seq++,
+				   (unsigned long) msg_imap->num);
 	  CHECK_ERROR (f_imap, status);
-	  MAILBOX_DEBUG0 (m_imap->mailbox, MU_DEBUG_PROT, f_imap->buffer);
+	  MU_DEBUG (m_imap->mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
 	  f_imap->state = IMAP_FETCH;
 	}
       status = fetch_operation (f_imap, msg_imap, 0, 0, 0);
@@ -1347,11 +1324,11 @@ imap_message_uid (mu_message_t msg, size_t *puid)
 	  *puid = msg_imap->uid;
 	  return 0;
 	}
-      status = imap_writeline (f_imap, "g%s FETCH %s UID\r\n",
-			       mu_umaxtostr (0, f_imap->seq++),
-			       mu_umaxtostr (1, msg_imap->num));
+      status = imap_writeline (f_imap, "g%lu FETCH %lu UID\r\n",
+			       (unsigned long) f_imap->seq++,
+			       (unsigned long) msg_imap->num);
       CHECK_ERROR (f_imap, status);
-      MAILBOX_DEBUG0 (m_imap->mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (m_imap->mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_FETCH;
     }
   status = fetch_operation (f_imap, msg_imap, 0, 0, 0);
@@ -1392,11 +1369,11 @@ imap_is_multipart (mu_message_t msg, int *ismulti)
 	  return 0;
 	}
       status = imap_writeline (f_imap,
-			       "g%s FETCH %s BODYSTRUCTURE\r\n",
-			       mu_umaxtostr (0, f_imap->seq++),
-			       mu_umaxtostr (1, msg_imap->num));
+			       "g%lu FETCH %lu BODYSTRUCTURE\r\n",
+			       (unsigned long) f_imap->seq++,
+			       (unsigned long) msg_imap->num);
       CHECK_ERROR (f_imap, status);
-      MAILBOX_DEBUG0 (m_imap->mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (m_imap->mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_FETCH;
     }
   status = fetch_operation (f_imap, msg_imap, 0, 0, 0);
@@ -1453,7 +1430,6 @@ imap_get_part (mu_message_t msg, size_t partno, mu_message_t *pmsg)
 	    {
 	      mu_header_t header;
 	      mu_message_get_header (message, &header);
-	      mu_header_set_get_value (header, NULL, message);
 	      mu_message_set_stream (message, NULL, msg_imap->parts[partno - 1]);
 	      /* mu_message_set_size (message, NULL, msg_imap->parts[partno - 1]); */
 	      msg_imap->parts[partno - 1]->message = message;
@@ -1477,31 +1453,32 @@ imap_envelope_sender (mu_envelope_t envelope, char *buffer, size_t buflen,
 {
   mu_message_t msg = mu_envelope_get_owner (envelope);
   mu_header_t header;
+  const char *sender;
   int status;
 
-  if (buflen == 0)
-    return 0;
-
   mu_message_get_header (msg, &header);
-  status = mu_header_get_value (header, MU_HEADER_SENDER, buffer, buflen, plen);
+  status = mu_header_sget_value (header, MU_HEADER_SENDER, &sender);
   if (status == EAGAIN)
     return status;
   else if (status != 0)
-    status = mu_header_get_value (header, MU_HEADER_FROM, buffer, buflen, plen);
+    status = mu_header_sget_value (header, MU_HEADER_FROM, &sender);
   if (status == 0)
     {
+      const char *email = NULL;
+      size_t len;
       mu_address_t address;
-      if (mu_address_create (&address, buffer) == 0)
+      if (mu_address_create (&address, sender) == 0)
 	{
-	  mu_address_get_email (address, 1, buffer, buflen, plen);
+	  if (mu_address_sget_email (address, 1, &email) == 0)
+	    len = mu_cpystr (buffer, email, buflen);
 	  mu_address_destroy (&address);
 	}
-    }
-  else if (status != EAGAIN)
-    {
-      strncpy (buffer, "Unknown", buflen)[buflen - 1] = '0';
+
+      if (!email)
+	return MU_ERR_NOENT;
+      
       if (plen)
-	*plen = strlen (buffer);
+	*plen = len;
     }
   return status;
 }
@@ -1532,11 +1509,11 @@ imap_envelope_date (mu_envelope_t envelope, char *buffer, size_t buflen,
       if (f_imap->state == IMAP_NO_STATE)
 	{
 	  status = imap_writeline (f_imap,
-				   "g%s FETCH %s INTERNALDATE\r\n",
-				   mu_umaxtostr (0, f_imap->seq++),
-				   mu_umaxtostr (1, msg_imap->num));
+				   "g%lu FETCH %lu INTERNALDATE\r\n",
+				   (unsigned long) f_imap->seq++,
+				   (unsigned long) msg_imap->num);
 	  CHECK_ERROR (f_imap, status);
-	  MAILBOX_DEBUG0 (m_imap->mailbox, MU_DEBUG_PROT, f_imap->buffer);
+	  MU_DEBUG (m_imap->mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
 	  f_imap->state = IMAP_FETCH;
 	}
       status = fetch_operation (f_imap, msg_imap, datebuf,
@@ -1603,11 +1580,11 @@ imap_attr_get_flags (mu_attribute_t attribute, int *pflags)
 
   if (f_imap->state == IMAP_NO_STATE)
     {
-      status = imap_writeline (f_imap, "g%s FETCH %s FLAGS\r\n",
-			       mu_umaxtostr (0, f_imap->seq++),
-			       mu_umaxtostr (1, msg_imap->num));
+      status = imap_writeline (f_imap, "g%lu FETCH %lu FLAGS\r\n",
+			       (unsigned long) f_imap->seq++,
+			       (unsigned long) msg_imap->num);
       CHECK_ERROR (f_imap, status);
-      MAILBOX_DEBUG0 (m_imap->mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (m_imap->mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_FETCH;
     }
   status = fetch_operation (f_imap, msg_imap, NULL, 0, NULL);
@@ -1659,13 +1636,13 @@ imap_attr_set_flags (mu_attribute_t attribute, int flag)
 	  free (abuf);
 	  return 0;
 	}
-      status = imap_writeline (f_imap, "g%s STORE %s +FLAGS.SILENT (%s)\r\n",
-			       mu_umaxtostr (0, f_imap->seq++),
-			       mu_umaxtostr (1, msg_imap->num),
+      status = imap_writeline (f_imap, "g%lu STORE %lu +FLAGS.SILENT (%s)\r\n",
+			       (unsigned long) f_imap->seq++,
+			       (unsigned long) msg_imap->num,
 			       abuf);
       free (abuf);
       CHECK_ERROR (f_imap, status);
-      MAILBOX_DEBUG0 (m_imap->mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (m_imap->mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
       msg_imap->flags |= flag;
       f_imap->state = IMAP_FETCH;
     }
@@ -1708,13 +1685,13 @@ imap_attr_unset_flags (mu_attribute_t attribute, int flag)
 	  free (abuf);
 	  return 0;
 	}
-      status = imap_writeline (f_imap, "g%s STORE %s -FLAGS.SILENT (%s)\r\n",
-			       mu_umaxtostr (0, f_imap->seq++),
-			       mu_umaxtostr (1, msg_imap->num),
+      status = imap_writeline (f_imap, "g%lu STORE %lu -FLAGS.SILENT (%s)\r\n",
+			       (unsigned long) f_imap->seq++,
+			       (unsigned long) msg_imap->num,
 			       abuf);
       free (abuf);
       CHECK_ERROR (f_imap, status);
-      MAILBOX_DEBUG0 (m_imap->mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (m_imap->mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
       msg_imap->flags &= ~flag;
       f_imap->state = IMAP_FETCH;
     }
@@ -1723,144 +1700,8 @@ imap_attr_unset_flags (mu_attribute_t attribute, int flag)
 
 /* Header.  */
 static int
-imap_header_get_value (mu_header_t header, const char *field, char * buffer,
-		       size_t buflen, size_t *plen)
-{
-  mu_message_t msg = mu_header_get_owner (header);
-  msg_imap_t msg_imap = mu_message_get_owner (msg);
-  m_imap_t m_imap = msg_imap->m_imap;
-  f_imap_t f_imap = m_imap->f_imap;
-  int status;
-  size_t len = 0;
-  char *value;
-
-  /* Select first.  */
-  status = imap_messages_count (m_imap->mailbox, NULL);
-  if (status != 0)
-    return status;
-
-  /* Hack, if buffer == NULL they want to know how big is the field value,
-     Unfortunately IMAP does not say, so we take a guess hoping that the
-     value will not be over 1024.  */
-  if (buffer == NULL || buflen == 0)
-    len = 1024;
-  else
-    len = strlen (field) + buflen + 4;
-
-  if (f_imap->state == IMAP_NO_STATE)
-    {
-      /* Select first.  */
-      status = imap_messages_count (m_imap->mailbox, NULL);
-      if (status != 0)
-	return status;
-      status = imap_writeline (f_imap,
-			       "g%s FETCH %s BODY.PEEK[HEADER.FIELDS (%s)]<0.%s>\r\n",
-			       mu_umaxtostr (0, f_imap->seq++),
-			       mu_umaxtostr (1, msg_imap->num),
-			       field,
-			       mu_umaxtostr (2, len));
-      CHECK_ERROR (f_imap, status);
-      MAILBOX_DEBUG0 (m_imap->mailbox, MU_DEBUG_PROT, f_imap->buffer);
-      f_imap->state = IMAP_FETCH;
-
-    }
-
-  value = calloc (len, sizeof (*value));
-  status = fetch_operation (f_imap, msg_imap, value, len, &len);
-  if (status == 0)
-    {
-      char *colon;
-      /* The field-matching is case-insensitive.  In all cases, the
-	 delimiting newline between the header and the body is always
-	 included. Nuke it  */
-      if (len)
-	value[len - 1] = '\0';
-
-      /* Move pass the field-name.  */
-      colon = strchr (value, ':');
-      if (colon)
-	{
-	  colon++;
-	  if (*colon == ' ')
-	    colon++;
-	}
-      else
-	colon = value;
-
-      while (*colon && colon[strlen (colon) - 1] == '\n')
-	colon[strlen (colon) - 1] = '\0';
-
-      if (buffer && buflen)
-	{
-	  strncpy (buffer, colon, buflen);
-	  buffer[buflen - 1] = '\0';
-	}
-      len = strlen (buffer);
-      if (plen)
-	*plen = len;
-      if (len == 0)
-	status = MU_ERR_NOENT;
-    }
-  free (value);
-  return status;
-}
-
-static int
-imap_header_get_fvalue (mu_header_t header, const char *field, char * buffer,
-			size_t buflen, size_t *plen)
-{
-  mu_message_t msg = mu_header_get_owner (header);
-  msg_imap_t msg_imap = mu_message_get_owner (msg);
-  m_imap_t m_imap = msg_imap->m_imap;
-  f_imap_t f_imap = m_imap->f_imap;
-  int status;
-  size_t len = 0;
-  char *value;
-
-  /* Select first.  */
-  status = imap_messages_count (m_imap->mailbox, NULL);
-  if (status != 0)
-    return status;
-
-  /* Do we all ready have the headers.  */
-  if (msg_imap->fheader)
-    return mu_header_get_value (msg_imap->fheader, field, buffer, buflen, plen);
-
-  /* We are caching the must use headers.  */
-  if (f_imap->state == IMAP_NO_STATE)
-    {
-      /* Select first.  */
-      status = imap_messages_count (m_imap->mailbox, NULL);
-      if (status != 0)
-        return status;
-      status = imap_writeline (f_imap,
-                               "g%s FETCH %s (FLAGS RFC822.SIZE BODY.PEEK[HEADER.FIELDS (%s)])\r\n",
-                               mu_umaxtostr (0, f_imap->seq++),
-			       mu_umaxtostr (1, msg_imap->num),
-			       MU_IMAP_CACHE_HEADERS);
-      CHECK_ERROR (f_imap, status);
-      MAILBOX_DEBUG0 (m_imap->mailbox, MU_DEBUG_PROT, f_imap->buffer);
-      f_imap->state = IMAP_FETCH;
-
-    }
-
-  /* Should be enough for our needs.  */
-  len = 2048;
-  value = calloc (len, sizeof *value);
-  status = fetch_operation (f_imap, msg_imap, value, len, &len);
-  if (status == 0)
-    {
-      status = mu_header_create (&msg_imap->fheader, value, len, NULL);
-      if (status == 0)
-	status =  mu_header_get_value (msg_imap->fheader, field, buffer,
-				    buflen, plen);
-    }
-  free (value);
-  return status;
-}
-
-static int
-imap_header_read (mu_header_t header, char *buffer, size_t buflen, mu_off_t offset,
+imap_header_read (mu_header_t header, char *buffer,
+		  size_t buflen, mu_off_t offset,
 		  size_t *plen)
 {
   mu_message_t msg = mu_header_get_owner (header);
@@ -1901,26 +1742,26 @@ imap_header_read (mu_header_t header, char *buffer, size_t buflen, mu_off_t offs
         {
           char *section = section_name (msg_imap);
           status = imap_writeline (f_imap,
-                                   "g%s FETCH %s BODY.PEEK[%s.MIME]<%s.%s>\r\n",
-                                   mu_umaxtostr (0, f_imap->seq++),
-				   mu_umaxtostr (1, msg_imap->num),
+                                   "g%lu FETCH %lu BODY.PEEK[%s.MIME]<%lu.%lu>\r\n",
+                                   (unsigned long) f_imap->seq++,
+				   (unsigned long) msg_imap->num,
                                    (section) ? section : "",
-                                   mu_umaxtostr (2, offset +
-						 msg_imap->header_lines),
-				   mu_umaxtostr (3, buflen));
+                                   (unsigned long) (offset +
+						    msg_imap->header_lines),
+				   (unsigned long) buflen);
           if (section)
             free (section);
         }
       else
         status = imap_writeline (f_imap,
-                                 "g%s FETCH %s BODY.PEEK[HEADER]<%s.%s>\r\n",
-                                 mu_umaxtostr (0, f_imap->seq++),
-				 mu_umaxtostr (1, msg_imap->num),
-                                 mu_umaxtostr (2, offset +
-					       msg_imap->header_lines),
-				 mu_umaxtostr (3, buflen));
+                                 "g%lu FETCH %lu BODY.PEEK[HEADER]<%lu.%lu>\r\n",
+                                 (unsigned long) f_imap->seq++,
+				 (unsigned long) msg_imap->num,
+                                 (unsigned long) (offset +
+						  msg_imap->header_lines),
+				 (unsigned long) buflen);
       CHECK_ERROR (f_imap, status);
-      MAILBOX_DEBUG0 (m_imap->mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (m_imap->mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_FETCH;
 
     }
@@ -2016,26 +1857,26 @@ imap_body_read (mu_stream_t stream, char *buffer, size_t buflen,
         {
           char *section = section_name (msg_imap);
           status = imap_writeline (f_imap,
-                                   "g%s FETCH %s BODY.PEEK[%s]<%s.%s>\r\n",
-                                   mu_umaxtostr (0, f_imap->seq++),
-				   mu_umaxtostr (1, msg_imap->num),
+                                   "g%lu FETCH %lu BODY.PEEK[%s]<%lu.%lu>\r\n",
+                                   (unsigned long) f_imap->seq++,
+				   (unsigned long) msg_imap->num,
                                    (section) ? section: "",
-                                   mu_umaxtostr (2, offset +
-						 msg_imap->body_lines),
-				   mu_umaxtostr (3, buflen));
+                                   (unsigned long) (offset +
+						    msg_imap->body_lines),
+				   (unsigned long) buflen);
           if (section)
             free (section);
         }
       else
         status = imap_writeline (f_imap,
-                                 "g%s FETCH %s BODY.PEEK[TEXT]<%s.%s>\r\n",
-                                 mu_umaxtostr (0, f_imap->seq++),
-				 mu_umaxtostr (1, msg_imap->num),
-                                 mu_umaxtostr (2, offset +
-					       msg_imap->body_lines),
-				 mu_umaxtostr (3, buflen));
+                                 "g%lu FETCH %lu BODY.PEEK[TEXT]<%lu.%lu>\r\n",
+                                 (unsigned long) f_imap->seq++,
+				 (unsigned long) msg_imap->num,
+                                 (unsigned long) (offset +
+						  msg_imap->body_lines),
+				 (unsigned long) buflen);
       CHECK_ERROR (f_imap, status);
-      MAILBOX_DEBUG0 (m_imap->mailbox, MU_DEBUG_PROT, f_imap->buffer);
+      MU_DEBUG (m_imap->mailbox->debug, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_FETCH;
 
     }
@@ -2091,8 +1932,8 @@ fetch_operation (f_imap_t f_imap, msg_imap_t msg_imap, char *buffer,
       status = imap_parse (f_imap);
       CHECK_EAGAIN (f_imap, status);
       if (f_imap->selected)
-	MAILBOX_DEBUG0 (f_imap->selected->mailbox, MU_DEBUG_PROT,
-			f_imap->buffer);
+	MU_DEBUG (f_imap->selected->mailbox->debug, MU_DEBUG_PROT,
+	   	  f_imap->buffer);
 
     default:
       break;
@@ -2206,7 +2047,7 @@ flags_to_string (char **pbuf, int flag)
 static int
 add_number (char **pset, size_t start, size_t end)
 {
-  char buf[2*UINTMAX_STRSIZE_BOUND+2];
+  char *buf = NULL;
   char *set;
   char *tmp;
   size_t set_len = 0;
@@ -2221,13 +2062,12 @@ add_number (char **pset, size_t start, size_t end)
 
   /* We had a previous seqence.  */
   if (start == 0)
-    *buf = '\0';
+    /* nothing */;
   else if (start != end)
-    snprintf (buf, sizeof buf, "%s:%s",
-	      mu_umaxtostr (0, start),
-	      mu_umaxtostr (1, end));
+    mu_asprintf (&buf, "%lu:%lu",
+		  (unsigned long) start, (unsigned long) end);
   else
-    snprintf (buf, sizeof buf, "%s", mu_umaxtostr (0, start));
+    mu_asprintf (&buf, "%lu", (unsigned long) start);
 
   if (set_len)
     tmp = realloc (set, set_len + strlen (buf) + 2 /* null and comma */);
@@ -2237,6 +2077,7 @@ add_number (char **pset, size_t start, size_t end)
   if (tmp == NULL)
     {
       free (set);
+      free (buf);
       return ENOMEM;
     }
   set = tmp;
@@ -2245,7 +2086,8 @@ add_number (char **pset, size_t start, size_t end)
   if (set_len)
     strcat (set, ",");
   strcat (set, buf);
-
+  free (buf);
+  
   *pset = set;
   return 0;
 }
