@@ -1,10 +1,11 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2005  Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2005, 2007, 2010 Free Software Foundation,
+   Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,8 +34,9 @@ struct _mu_observer
 {
   int flags;
   void *owner;
-  int (*_action)  (mu_observer_t, size_t);
-  int (*_destroy) (mu_observer_t);
+  int (*_action)  (mu_observer_t, size_t, void *, void *);
+  void *_action_data;
+  int (*_destroy) (mu_observer_t, void *data);
 };
 
 struct _mu_observable
