@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2003, 
-   2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2003, 2004, 2005, 2006, 2007, 2008,
+   2009, 2010 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -24,7 +24,8 @@
 #endif
 
 #include <mbox0.h>
-#include <mu_umaxtostr.h>
+#include <mailutils/cstr.h>
+#include <mailutils/io.h>
 
 #define ATTRIBUTE_IS_DELETED(flag)        (flag & MU_ATTRIBUTE_DELETED)
 #define ATTRIBUTE_IS_EQUAL(flag1, flag2)  (flag1 == flag2)
@@ -350,7 +351,7 @@ mbox_is_updated (mu_mailbox_t mailbox)
       mu_observable_notify (mailbox->observable, MU_EVT_MAILBOX_CORRUPT,
 			    mailbox);
       /* And be verbose.  ? */
-      mu_error (_("* BAD : Mailbox corrupted, shrank in size"));
+      mu_diag_output (MU_DIAG_EMERG, _("mailbox corrupted, shrank in size"));
       /* FIXME: should I crash.  */
       return 0;
     }
@@ -465,7 +466,7 @@ mbox_expunge0 (mu_mailbox_t mailbox, int remove_deleted)
     {
       if (tmpmboxname)
 	free (tmpmboxname);
-      mu_error (_("Failed to create temporary file when expunging"));
+      mu_error (_("failed to create temporary file when expunging"));
       return errno;
     }
 
@@ -531,7 +532,7 @@ mbox_expunge0 (mu_mailbox_t mailbox, int remove_deleted)
       mu_mailbox_destroy (&tmpmailbox);
       remove (tmpmboxname);
       free (tmpmboxname);
-      mu_error (_("Failed to grab the lock: %s"), mu_strerror (status));
+      mu_error (_("failed to grab the lock: %s"), mu_strerror (status));
       return status;
     }
 
@@ -583,7 +584,7 @@ mbox_expunge0 (mu_mailbox_t mailbox, int remove_deleted)
 	      status = mbox_get_message (mailbox, i + 1, &msg);
 	      if (status != 0)
 		{
-		  mu_error (_("Error expunging:%d: %s"), __LINE__,
+		  mu_error (_("error expunging:%d: %s"), __LINE__,
 			    mu_strerror (status));
 		  goto bailout0;
 		}
@@ -592,7 +593,7 @@ mbox_expunge0 (mu_mailbox_t mailbox, int remove_deleted)
 					 &total, 1, (i == save_imapbase));
 	  if (status != 0)
 	    {
-	      mu_error (_("Error expunging:%d: %s"), __LINE__,
+	      mu_error (_("error expunging:%d: %s"), __LINE__,
 		        mu_strerror (status));
 	      goto bailout0;
 	    }
@@ -615,7 +616,7 @@ mbox_expunge0 (mu_mailbox_t mailbox, int remove_deleted)
 		  || (status = mu_stream_write (tmpmailbox->stream, buffer, n,
 					     total, &n) != 0))
 		{
-		  mu_error (_("Error expunging:%d: %s"), __LINE__,
+		  mu_error (_("error expunging:%d: %s"), __LINE__,
 			    mu_strerror (status));
 		  goto bailout0;
 		}
@@ -627,7 +628,7 @@ mbox_expunge0 (mu_mailbox_t mailbox, int remove_deleted)
 	  status = mu_stream_write (tmpmailbox->stream, "\n", 1, total, &n);
 	  if (status != 0)
 	    {
-	      mu_error (_("Error expunging:%d: %s"), __LINE__,
+	      mu_error (_("error expunging:%d: %s"), __LINE__,
 		        mu_strerror (status));
 	      goto bailout0;
 	    }
@@ -657,7 +658,7 @@ mbox_expunge0 (mu_mailbox_t mailbox, int remove_deleted)
 				       total, &n);
 		if (status != 0)
 		  {
-		    mu_error (_("Error expunging:%d: %s"), __LINE__,
+		    mu_error (_("error expunging:%d: %s"), __LINE__,
 			      mu_strerror (status));
 		    goto bailout0;
 		  }
@@ -668,7 +669,7 @@ mbox_expunge0 (mu_mailbox_t mailbox, int remove_deleted)
 	else if (len < 0)
 	  {
 	    /* Corrupted mailbox.  */
-	    mu_error (_("Error expunging:%d: %s"), __LINE__,
+	    mu_error (_("error expunging:%d: %s"), __LINE__,
 		      mu_strerror (status));
 	    goto bailout0;
 	  }
@@ -689,7 +690,7 @@ mbox_expunge0 (mu_mailbox_t mailbox, int remove_deleted)
 	  status = mu_stream_write (mailbox->stream, buffer, n, offset, &n);
 	  if (status != 0)
 	    {
-	      mu_error (_("Error expunging:%d: %s"), __LINE__,
+	      mu_error (_("error expunging:%d: %s"), __LINE__,
 		        mu_strerror (status));
 	      goto bailout;
 	    }
@@ -703,7 +704,7 @@ mbox_expunge0 (mu_mailbox_t mailbox, int remove_deleted)
   status = mu_stream_truncate (mailbox->stream, total + marker);
   if (status != 0)
     {
-      mu_error (_("Error expunging:%d: %s"), __LINE__,
+      mu_error (_("error expunging:%d: %s"), __LINE__,
 	        mu_strerror (status));
       goto bailout;
     }
@@ -805,12 +806,7 @@ static int
 mbox_message_qid (mu_message_t msg, mu_message_qid_t *pqid)
 {
   mbox_message_t mum = mu_message_get_owner (msg);
-  char buf[UINTMAX_STRSIZE_BOUND];
-  const char *p = umaxtostr (mum->header_from, buf);
-  *pqid = strdup (p);
-  if (*pqid == NULL)
-    return ENOMEM;
-  return 0;
+  return mu_asprintf (pqid, "%lu", (unsigned long) mum->header_from);
 }
 
 static int
@@ -1198,8 +1194,8 @@ mbox_get_message (mu_mailbox_t mailbox, size_t msgno, mu_message_t *pmsg)
       return 0;
     }
 
-  MU_DEBUG2 (mailbox->debug, MU_DEBUG_TRACE1, "mbox_get_message (%s, %d)\n",
-		  mud->name, msgno);
+  MU_DEBUG2 (mailbox->debug, MU_DEBUG_TRACE1, "mbox_get_message (%s, %lu)\n",
+	     mud->name, (unsigned long) msgno);
 
   return new_message (mailbox, mum, pmsg);
 }
@@ -1267,7 +1263,7 @@ mbox_append_message (mu_mailbox_t mailbox, mu_message_t msg)
 {
   int status = 0;
   mbox_data_t mud = mailbox->data;
-  mu_off_t size;
+  mu_off_t qid;
   
   if (msg == NULL || mud == NULL)
     return EINVAL;
@@ -1288,9 +1284,11 @@ mbox_append_message (mu_mailbox_t mailbox, mu_message_t msg)
 
     default:
       {
+	mu_off_t size;
 	/* Move to the end of the file, not necesary if _APPEND mode.  */
 	if ((status = mu_stream_size (mailbox->stream, &size)) != 0
-	    || (status = mbox_append_message0 (mailbox, msg,
+	    || (qid = size,
+	        status = mbox_append_message0 (mailbox, msg,
 					       &size, 0, 0)) != 0)
 	  {
 	    if (status != EAGAIN)
@@ -1303,9 +1301,10 @@ mbox_append_message (mu_mailbox_t mailbox, mu_message_t msg)
 
   if (mailbox->observable)
     {
-      char buf[UINTMAX_STRSIZE_BOUND];
-      mu_observable_notify (mailbox->observable, MU_EVT_MESSAGE_APPEND,
-			    umaxtostr (size, buf)); 
+      char *buf = NULL;
+      mu_asprintf (&buf, "%lu", (unsigned long) qid);
+      mu_observable_notify (mailbox->observable, MU_EVT_MESSAGE_APPEND, buf);
+      free (buf);
     }
   
   return 0;
@@ -1554,11 +1553,11 @@ mbox_append_message0 (mu_mailbox_t mailbox, mu_message_t msg, mu_off_t *psize,
 	       FIXME:
 	       - We have a problem here the header may not fit the buffer.
 	       - Should  we skip the IMAP "X-Status"? */
-	    if ((strncasecmp (buffer, "Status", 6) == 0)
-		|| (strncasecmp (buffer, "X-IMAPbase", 10) == 0)
+	    if ((mu_c_strncasecmp (buffer, "Status", 6) == 0)
+		|| (mu_c_strncasecmp (buffer, "X-IMAPbase", 10) == 0)
 		/* FIXME: isn't the length of "X-UID" 5, not 4? And
 		 this will match X-UID and X-UIDL, is this intended? */
-		|| (strncasecmp (buffer, "X-UID", 4) == 0
+		|| (mu_c_strncasecmp (buffer, "X-UID", 4) == 0
 		    && (buffer[5] == ':' || buffer[5] == ' '
 			|| buffer[5] == '\t')))
 	      continue;
@@ -1589,14 +1588,19 @@ mbox_append_message0 (mu_mailbox_t mailbox, mu_message_t msg, mu_off_t *psize,
       case MBOX_STATE_APPEND_ATTRIBUTE:
       /* Put the new attributes.  */
       {
-	char abuf[64];
+#define STATUS_PREFIX_LEN (sizeof(MU_HEADER_STATUS) - 1 + 2)
+	char abuf[STATUS_PREFIX_LEN + MU_STATUS_BUF_SIZE + 1];
 	size_t na = 0;
 	mu_attribute_t attr = NULL;
-	abuf[0] = '\0';
-	mu_message_get_attribute (msg, &attr);
-	mu_attribute_to_string (attr, abuf, sizeof(abuf), &na);
 
-	status = mu_stream_write (mailbox->stream, abuf, na, *psize, &n);
+	strcpy(abuf, MU_HEADER_STATUS);
+	strcat(abuf, ": ");
+	mu_message_get_attribute (msg, &attr);
+	mu_attribute_to_string (attr, abuf + STATUS_PREFIX_LEN, 
+	                        sizeof(abuf) - STATUS_PREFIX_LEN - 1, &na);
+	strcat (abuf, "\n");
+	na = strlen (abuf);
+	mu_stream_write (mailbox->stream, abuf, na, *psize, &n);
 	if (status != 0)
 	  break;
 	*psize += n;
