@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2007, 2010 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -39,21 +40,12 @@ mail_setenv (int argc, char **argv)
       /* Note: POSIX requires that the argument to putenv become part
 	 of the environment itself, hence the memory allocation. */
 
-      for (i = 1; i < argc;)
+      for (i = 1; i < argc; i++)
 	{
-	  char *p;
-	  
-	  if (i+1 < argc && argv[i+1][0] == '=')
-	    {
-	      asprintf (&p, "%s=%s", argv[i], argv[i+2]);
-	      i += 3;
-	    }
-	  else
-	    {
-	      p = argv[i];
-	      i++;
-	    }
-	  putenv (p);
+	  char *value = strchr (argv[i], '=');
+	  if (value)
+	    *value++ = 0;
+	  setenv (argv[i], value ? value : "", 1);
 	}
     }
   return 0;
