@@ -1,6 +1,6 @@
 /* argcv.c - simple functions for parsing input based on whitespace
-   Copyright (C) 1999, 2000, 2001, 2003, 2004, 
-   2005, 2006 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2003, 2004, 2005, 2006, 2010 Free
+   Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -159,7 +159,7 @@ argcv_scan (struct argcv_info *ap)
   return ap->save;
 }
 
-static char quote_transtab[] = "\\\\a\ab\bf\fn\nr\rt\t";
+static char quote_transtab[] = "\\\\\"\"a\ab\bf\fn\nr\rt\tv\v";
 
 int
 argcv_unquote_char (int c)
