@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2004, 
-   2005, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2004, 2005, 2006, 2007, 2008,
+   2010 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -49,7 +49,10 @@ struct _file_stream
   mu_off_t offset;
   int tempfile;
   char *filename;
+  /* The following three members are used for stdio streams only. */
+  int size_computed;
   mu_stream_t cache;
+  mu_off_t size;
 };
 
 static void
@@ -208,7 +211,7 @@ _stdin_file_read (mu_stream_t stream, char *optr, size_t osize,
   int status = 0;
   size_t nbytes;
   struct _file_stream *fs = mu_stream_get_owner (stream);
-  int fs_offset = fs->offset;
+  mu_off_t fs_offset = fs->offset;
 
   if (offset < fs_offset)
     return mu_stream_read (fs->cache, optr, osize, offset, pnbytes);
@@ -269,7 +272,7 @@ _stdin_file_readline (mu_stream_t stream, char *optr, size_t osize,
   int status;
   size_t nbytes;
   struct _file_stream *fs = mu_stream_get_owner (stream);
-  int fs_offset = fs->offset;
+  mu_off_t fs_offset = fs->offset;
   
   if (offset < fs->offset)
     return mu_stream_readline (fs->cache, optr, osize, offset, pnbytes);
@@ -293,6 +296,31 @@ _stdin_file_readline (mu_stream_t stream, char *optr, size_t osize,
   return status;
 }
 
+/* Used only if stream->cache is not NULL */ 
+static int
+_stdin_file_size (mu_stream_t stream, mu_off_t *psize)
+{
+  struct _file_stream *fs = mu_stream_get_owner (stream);
+
+  if (!fs->size_computed)
+    {
+      char buf[512];
+      mu_off_t fs_offset = fs->offset;
+      size_t n;
+      int status;
+      
+      /* Fill in the cache */ 
+      while ((status = mu_stream_read (stream, buf, sizeof (buf),
+				       fs_offset, &n)) == 0
+	     && n > 0)
+	fs_offset += n;
+      fs->size = fs_offset;
+      fs->size_computed = 1;
+    }
+  *psize = fs->size;
+  return 0;
+}
+
 static int
 _stdout_file_write (mu_stream_t stream, const char *iptr, size_t isize,
 		    mu_off_t offset, size_t *nbytes)
@@ -418,25 +446,24 @@ _file_open (mu_stream_t stream)
   const char *mode;
   char* filename = 0;
   int flags = 0;
-
-  assert(fs);
-
+  
+  if (!fs || !fs->filename)
+    return EINVAL;
+  
   filename = fs->filename;
 
-  assert(filename);
-
   if (fs->file)
     {
       fclose (fs->file);
       fs->file = NULL;
     }
 
-  mu_stream_get_flags(stream, &flags);
+  mu_stream_get_flags (stream, &flags);
 
   /* Map the flags to the system equivalent.  */
   if (flags & MU_STREAM_WRITE && flags & MU_STREAM_READ)
     return EINVAL;
-  else if (flags & MU_STREAM_WRITE)
+  else if (flags & (MU_STREAM_WRITE|MU_STREAM_APPEND))
     flg = O_WRONLY;
   else if (flags & MU_STREAM_RDWR)
     flg = O_RDWR;
@@ -453,14 +480,15 @@ _file_open (mu_stream_t stream)
   if (flags & MU_STREAM_CREAT)
     {
       /* First see if the file already exists.  */
-      fd = open(filename, flg);
+      fd = open (filename, flg);
       if (fd == -1)
 	{
 	  /* Oops bail out.  */
 	  if (errno != ENOENT)
 	    return errno;
 	  /* Race condition here when creating the file ??.  */
-	  fd = open(filename, flg|O_CREAT|O_EXCL, 0600);
+	  fd = open (filename, flg|O_CREAT|O_EXCL,
+		     0600 | mu_stream_flags_to_mode (flags, 0));
 	  if (fd < 0)
 	    return errno;
 	}
@@ -474,16 +502,15 @@ _file_open (mu_stream_t stream)
 
   /* We have to make sure that We did not open
      a symlink. From Casper D. in bugtraq.  */
-  if ((flg & MU_STREAM_CREAT) ||
-      (flg & MU_STREAM_RDWR) ||
-      (flg & MU_STREAM_WRITE))
+  if (flg & (MU_STREAM_CREAT | MU_STREAM_RDWR
+	     | MU_STREAM_WRITE | MU_STREAM_APPEND))
     {
       struct stat fdbuf, filebuf;
 
       /* The next two stats should never fail.  */
-      if (fstat(fd, &fdbuf) == -1)
+      if (fstat (fd, &fdbuf) == -1)
 	return errno;
-      if (lstat(filename, &filebuf) == -1)
+      if (lstat (filename, &filebuf) == -1)
 	return errno;
 
       /* Now check that: file and fd reference the same file,
@@ -495,7 +522,7 @@ _file_open (mu_stream_t stream)
 	      || filebuf.st_nlink != 1
 	      || (fdbuf.st_mode & S_IFMT) != S_IFREG))
 	{
-	  mu_error (_("%s must be a plain file with one link\n"), filename);
+	  mu_error (_("%s must be a plain file with one link"), filename);
 	  close (fd);
 	  return EINVAL;
 	}
@@ -546,8 +573,8 @@ mu_file_stream_create (mu_stream_t *stream, const char* filename, int flags)
   ret = mu_stream_create (stream, flags|MU_STREAM_NO_CHECK, fs);
   if (ret != 0)
     {
-      free (fs);
       free (fs->filename);
+      free (fs);
       return ret;
     }
 
@@ -653,6 +680,7 @@ mu_stdio_stream_create (mu_stream_t *stream, FILE *file, int flags)
       mu_stream_set_read (*stream, _stdin_file_read, fs);
       mu_stream_set_readline (*stream, _stdin_file_readline, fs);
       mu_stream_set_write (*stream, _stdout_file_write, fs);
+      mu_stream_set_size (*stream, _stdin_file_size, fs);
     }
   else
     {
@@ -679,7 +707,7 @@ struct _prog_stream
   pid_t pid;
   int status;
   pid_t writer_pid;
-  size_t argc;
+  int argc;
   char **argv;
   mu_stream_t in, out;
 
@@ -706,28 +734,6 @@ _prog_stream_unregister (struct _prog_stream *stream)
   mu_list_remove (prog_stream_list, stream);
 }
 
-static int
-_prog_waitpid (void *item, void *data MU_ARG_UNUSED)
-{
-  struct _prog_stream *str = item;
-  int status;
-  if (str->pid > 0)
-    {
-      if (waitpid (str->pid, &str->status, WNOHANG) == str->pid)
-	str->pid = -1;
-    }
-  if (str->writer_pid > 0)
-    waitpid (str->writer_pid, &status, WNOHANG);
-  return 0;
-}
-
-static void
-_prog_stream_wait (struct _prog_stream *fs)
-{
-  if (fs->pid > 0)
-    waitpid (fs->pid, &fs->status, 0);
-}
-
 #if defined (HAVE_SYSCONF) && defined (_SC_OPEN_MAX)
 # define getmaxfd() sysconf (_SC_OPEN_MAX)
 #elif defined (HAVE_GETDTABLESIZE)
@@ -842,24 +848,35 @@ start_program_filter (pid_t *pid, int *p, int argc, char **argv,
   return rc;
 }
 
+static void
+_prog_wait (pid_t pid, int *pstatus)
+{
+  if (pid > 0)
+    {
+      pid_t t;
+      do
+	t = waitpid (pid, pstatus, 0);
+      while (t == -1 && errno == EINTR);
+    }
+}
+
 static void
 _prog_destroy (mu_stream_t stream)
 {
   struct _prog_stream *fs = mu_stream_get_owner (stream);
+  int status;
+    
   mu_argcv_free (fs->argc, fs->argv);
   if (fs->in)
     mu_stream_destroy (&fs->in, mu_stream_get_owner (fs->in));
   if (fs->out)
     mu_stream_destroy (&fs->out, mu_stream_get_owner (fs->out));
-  if (fs->pid > 0)
-    {
-      kill (fs->pid, SIGTERM);
-      mu_list_do (prog_stream_list, _prog_waitpid, NULL);
-      kill (fs->pid, SIGKILL);
-      if (fs->writer_pid > 0)
-	kill (fs->writer_pid, SIGKILL);
-      mu_list_do (prog_stream_list, _prog_waitpid, NULL);
-    }
+  
+  _prog_wait (fs->pid, &fs->status);
+  fs->pid = -1;
+  _prog_wait (fs->writer_pid, &status);
+  fs->writer_pid = -1;
+  
   _prog_stream_unregister (fs);
 }
 
@@ -867,6 +884,7 @@ static int
 _prog_close (mu_stream_t stream)
 {
   struct _prog_stream *fs = mu_stream_get_owner (stream);
+  int status;
   
   if (!stream)
     return EINVAL;
@@ -877,7 +895,10 @@ _prog_close (mu_stream_t stream)
   mu_stream_close (fs->out);
   mu_stream_destroy (&fs->out, mu_stream_get_owner (fs->out));
 
-  _prog_stream_wait (fs);
+  _prog_wait (fs->pid, &fs->status);
+  fs->pid = -1;
+  _prog_wait (fs->writer_pid, &status);
+  fs->writer_pid = -1;
   
   mu_stream_close (fs->in);
   mu_stream_destroy (&fs->in, mu_stream_get_owner (fs->in));
@@ -1103,7 +1124,7 @@ mu_prog_stream_create (mu_stream_t *stream, const char *progname, int flags)
 
 int
 mu_filter_prog_stream_create (mu_stream_t *stream, const char *progname,
-			   mu_stream_t input)
+			      mu_stream_t input)
 {
   struct _prog_stream *fs;
   int rc = _prog_stream_create (&fs, stream, progname, MU_STREAM_RDWR);
