@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2004, 2005,
-   2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2004, 2005, 2007, 2010 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -274,15 +274,6 @@ qp_decode (const char *iptr, size_t isize, char *optr, size_t osize,
   return consumed - wscount;
 }
 
-#define SOFTBRK() \
-      /* check if we have enough room for the soft linebreak */\
-      if (*nbytes + 2 > osize) \
-  	break;\
-      *optr++ = '=';\
-      *optr++ = '\n';\
-      (*nbytes) += 2;\
-      *line_len = 0;
-
 static int
 qp_encode (const char *iptr, size_t isize, char *optr, size_t osize,
 	   size_t *nbytes, int *line_len)
