@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005, 2006, 2007, 
-   2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -117,7 +117,7 @@ static mh_msgset_t msgset;
 static mu_mailbox_t mbox;
 
 static int
-opt_handler (int key, char *arg, void *unused, struct argp_state *state)
+opt_handler (int key, char *arg, struct argp_state *state)
 {
   switch (key)
     {
@@ -204,17 +204,20 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
       
     case ARG_INPLACE:
+      mh_opt_notimpl_warning ("-inplace");
+      break;
+
     case ARG_WHATNOWPROC:
     case ARG_NOWHATNOWPROC:
-      argp_error (state, _("option is not yet implemented"));
-      exit (1);
-      
+      mh_opt_notimpl ("-[no]whatnowproc");
+      break;
+ 
     case ARG_LICENSE:
       mh_license (argp_program_version);
       break;
 
     default:
-      return 1;
+      return ARGP_ERR_UNKNOWN;
     }
   return 0;
 }
@@ -389,12 +392,12 @@ finish_draft ()
       if (msgset.count == 1)
 	{
 	  fd.num = 0;
-	  str = _("Forwarded message\n");
+	  str = (char*) _("Forwarded message\n");
 	}
       else
 	{
 	  fd.num = 1;
-	  str = _("Forwarded messages\n");
+	  str = (char*) _("Forwarded messages\n");
 	}
   
       rc = mu_stream_sequential_write (stream, str, strlen (str));
@@ -406,9 +409,9 @@ finish_draft ()
       rc = mu_stream_sequential_write (stream, str, strlen (str));
       
       if (msgset.count == 1)
-	str = _("End of Forwarded message");
+	str = (char*) _("End of Forwarded message");
       else
-	str = _("End of Forwarded messages");
+	str = (char*) _("End of Forwarded messages");
       
       rc = mu_stream_sequential_write (stream, str, strlen (str));
     }
