@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 
-   2005, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2007, 2009,
+   2010 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 
 #include <mh.h>
 #include <mailutils/url.h>
+#include <mailutils/tls.h>
 #include <pwd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -32,18 +33,23 @@
 #include <fnmatch.h>
 
 char mh_list_format[] = 
-"%4(msg)%<(cur)+%| %>%<{replied}-%?{encrypted}E%| %>"
-"%02(mon{date})/%02(mday{date})"
-"%<{date} %|*%>"
-"%<(mymbox{from})%<{to}To:%14(friendly{to})%>%>"
-"%<(zero)%17(friendly{from})%>"
-"  %{subject}%<{body}<<%{body}>>%>";
+  "%4(msg)"
+  "%<(cur)+%| %>"
+  "%<{replied}-%?{encrypted}E%| %>"
+  "%02(mon{date})/%02(mday{date})"
+  "%<{date} %|*%>"
+  "%<(mymbox{from})%<{to}To:%14(decode(friendly{to}))%>%>"
+  "%<(zero)%17(decode(friendly{from}))%>"
+  "  %(decode{subject})%<{body}<<%{body}>>%>";
 
 void
 mh_init ()
 {
   /* Register all mailbox and mailer formats */
   mu_register_all_formats ();
+#ifdef WITH_TLS
+  mu_init_tls_libs ();
+#endif
 
   /* Read user's profile */
   mh_read_profile ();
@@ -67,14 +73,14 @@ mh_read_formfile (char *name, char **pformat)
 
   if (stat (name, &st))
     {
-      mu_error (_("Cannot stat format file %s: %s"), name, strerror (errno));
+      mu_error (_("cannot stat format file %s: %s"), name, strerror (errno));
       return -1;
     }
   
   fp = fopen (name, "r");
   if (!fp)
     {
-      mu_error (_("Cannot open format file %s: %s"), name, strerror (errno));
+      mu_error (_("cannot open format file %s: %s"), name, strerror (errno));
       return -1;
     }
 
@@ -109,7 +115,7 @@ mh_read_formfile (char *name, char **pformat)
 void
 mh_err_memory (int fatal)
 {
-  mu_error (_("Not enough memory"));
+  mu_error (_("not enough memory"));
   if (fatal)
     abort ();
 }
@@ -125,7 +131,7 @@ mh_get_my_name (char *name)
       struct passwd *pw = getpwuid (getuid ());
       if (!pw)
 	{
-	  mu_error (_("Cannot determine my username"));
+	  mu_error (_("cannot determine my username"));
 	  return;
 	}
       name = pw->pw_name;
@@ -143,7 +149,7 @@ emailcmp (char *pattern, char *name)
   p = strchr (pattern, '@');
   if (p)
     for (p++; *p; p++)
-      *p = toupper (*p);
+      *p = mu_toupper (*p);
 
   return fnmatch (pattern, name, 0);
 }
@@ -158,7 +164,7 @@ mh_is_my_name (const char *name)
   p = strchr (pname, '@');
   if (p)
     for (p++; *p; p++)
-      *p = toupper (*p);
+      *p = mu_toupper (*p);
   
   if (!my_email)
     mh_get_my_name (NULL);
@@ -176,7 +182,7 @@ mh_is_my_name (const char *name)
 	  for (p = nlist; rc == 0 && *p; p = end)
 	    {
 	      
-	      while (*p && isspace (*p))
+	      while (*p && mu_isspace (*p))
 		p++;
 
 	      end = strchr (p, ',');
@@ -191,7 +197,7 @@ mh_is_my_name (const char *name)
 		  end = p + len;
 		}
 
-	      while (len > 0 && isspace (p[len-1]))
+	      while (len > 0 && mu_isspace (p[len-1]))
 		len--;
 
 	      pat = xmalloc (len + 1);
@@ -228,13 +234,13 @@ make_dir_hier (const char *p, mode_t perm)
 	{
 	  if (errno != ENOENT)
 	    {
-	      mu_error (_("Cannot create directory %s: error accessing name component %s: %s"),
+	      mu_error (_("cannot create directory %s: error accessing name component %s: %s"),
 
 			p, dir, strerror (errno));
 	      rc = 1;
 	    }
 	  else if ((rc = mkdir (dir, perm)))
-	    mu_error (_("Cannot create directory %s: error creating name component %s: %s"),
+	    mu_error (_("cannot create directory %s: error creating name component %s: %s"),
 		      p, dir, mu_strerror (rc));
 	}
       *q = '/';
@@ -259,7 +265,7 @@ mh_makedir (const char *p)
     {
       rc = mkdir (p, perm);
       if (rc)
-	mu_error (_("Cannot create directory %s: %s"),
+	mu_error (_("cannot create directory %s: %s"),
 		  p, strerror (errno));
     }
 
@@ -282,6 +288,8 @@ mh_check_folder (const char *pathname, int confirm)
     {
       if (errno == ENOENT)
 	{
+	  /* TRANSLATORS: This is a question and will be followed
+	     by question mark on output. */
 	  if (!confirm || mh_getyn (_("Create folder \"%s\""), p))
 	    return mh_makedir (p);
 	  else
@@ -289,7 +297,7 @@ mh_check_folder (const char *pathname, int confirm)
 	}
       else
 	{
-	  mu_error (_("Cannot stat %s: %s"), p, strerror (errno));
+	  mu_diag_funcall (MU_DIAG_ERROR, "stat", p, errno);
 	  return 1;
 	}
     }
@@ -380,7 +388,7 @@ mh_audit_open (char *name, mu_mailbox_t mbox)
       asprintf (&p, "%s/%s", mu_folder_directory (), namep);
       if (!p)
 	{
-	  mu_error (_("Not enough memory"));
+	  mu_error (_("not enough memory"));
 	  exit (1);
 	}
       free (namep);
@@ -390,7 +398,7 @@ mh_audit_open (char *name, mu_mailbox_t mbox)
   fp = fopen (namep, "a");
   if (!fp)
     {
-      mu_error (_("Cannot open audit file %s: %s"), namep, strerror (errno));
+      mu_error (_("cannot open audit file %s: %s"), namep, strerror (errno));
       free (namep);
       return NULL;
     }
@@ -433,7 +441,7 @@ mh_open_folder (const char *folder, int create)
     
   if (mu_mailbox_create_default (&mbox, name))
     {
-      mu_error (_("Cannot create mailbox %s: %s"),
+      mu_error (_("cannot create mailbox %s: %s"),
 		name, strerror (errno));
       exit (1);
     }
@@ -443,7 +451,7 @@ mh_open_folder (const char *folder, int create)
   
   if (mu_mailbox_open (mbox, flags))
     {
-      mu_error (_("Cannot open mailbox %s: %s"), name, strerror (errno));
+      mu_error (_("cannot open mailbox %s: %s"), name, strerror (errno));
       exit (1);
     }
 
@@ -521,7 +529,8 @@ mh_iterate (mu_mailbox_t mbox, mh_msgset_t *msgset,
       num = msgset->list[i];
       if ((rc = mu_mailbox_get_message (mbox, num, &msg)) != 0)
 	{
-	  mu_error (_("Cannot get message %d: %s"), num, mu_strerror (rc));
+	  mu_error (_("cannot get message %lu: %s"),
+		    (unsigned long) num, mu_strerror (rc));
 	  return 1;
 	}
 
@@ -537,9 +546,9 @@ mh_spawnp (const char *prog, const char *file)
   int argc, i, rc, status;
   char **argv, **xargv;
 
-  if (mu_argcv_get (prog, "", "#", &argc, &argv))
+  if ((rc = mu_argcv_get (prog, "", "#", &argc, &argv)) != 0)
     {
-      mu_error (_("Cannot split line %s"), prog);
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_argcv_get", prog, rc);
       mu_argcv_free (argc, argv);
       return 1;
     }
@@ -591,7 +600,7 @@ mh_file_copy (const char *from, const char *to)
   if ((rc = mu_file_stream_create (&in, from, MU_STREAM_READ)) != 0
       || (rc = mu_stream_open (in)))
     {
-      mu_error (_("Cannot open input file `%s': %s"),
+      mu_error (_("cannot open input file `%s': %s"),
 		from, mu_strerror (rc));
       free (buffer);
       return 1;
@@ -600,7 +609,7 @@ mh_file_copy (const char *from, const char *to)
   if ((rc = mu_file_stream_create (&out, to, MU_STREAM_RDWR|MU_STREAM_CREAT)) != 0
       || (rc = mu_stream_open (out)))
     {
-      mu_error (_("Cannot open output file `%s': %s"),
+      mu_error (_("cannot open output file `%s': %s"),
 		to, mu_strerror (rc));
       free (buffer);
       mu_stream_close (in);
@@ -614,7 +623,7 @@ mh_file_copy (const char *from, const char *to)
     {
       if ((rc = mu_stream_sequential_write (out, buffer, rdsize)) != 0)
 	{
-	  mu_error (_("Write error on `%s': %s"),
+	  mu_error (_("write error on `%s': %s"),
 		    to, mu_strerror (rc));
 	  break;
 	}
@@ -640,20 +649,20 @@ _file_to_message (const char *file_name)
 
   if (stat (file_name, &st) < 0)
     {
-      mu_error (_("Cannot stat file %s: %s"), file_name, strerror (errno));
+      mu_diag_funcall (MU_DIAG_ERROR, "stat", file_name, errno);
       return NULL;
     }
   
   if ((rc = mu_file_stream_create (&instream, file_name, MU_STREAM_READ)))
     {
-      mu_error (_("Cannot create input stream (file %s): %s"),
+      mu_error (_("cannot create input stream (file %s): %s"),
 		file_name, mu_strerror (rc));
       return NULL;
     }
   
   if ((rc = mu_stream_open (instream)))
     {
-      mu_error (_("Cannot open input stream (file %s): %s"),
+      mu_error (_("cannot open input stream (file %s): %s"),
 		file_name, mu_strerror (rc));
       mu_stream_destroy (&instream, mu_stream_get_owner (instream));
       return NULL;
@@ -707,14 +716,20 @@ mh_real_install (char *name, int automode)
     {
       size_t n = 0;
       
+      /* TRANSLATORS: This is a question and will be followed
+	 by question mark on output. */
       if (mh_getyn_interactive (_("Do you need help")))
 	mh_install_help (mhdir);
 
+      /* TRANSLATORS: This is a question and will be followed
+	 by question mark on output. */
       if (!mh_getyn_interactive (_("Do you want the standard MH path \"%s\""), mhdir))
 	{
 	  int local;
 	  char *p;
 	  
+	  /* TRANSLATORS: This is a question and will be followed
+	     by question mark on output. */
 	  local = mh_getyn_interactive (_("Do you want a path below your login directory"));
 	  if (local)
 	    printf (_("What is the path? "));
@@ -747,7 +762,7 @@ mh_real_install (char *name, int automode)
   fp = fopen (name, "w");
   if (!fp)
     {
-      mu_error (_("Cannot open file %s: %s"), name, mu_strerror (errno));
+      mu_error (_("cannot open file %s: %s"), name, mu_strerror (errno));
       exit (1);
     }
   fprintf (fp, "Path: %s\n", mhdir);
@@ -783,7 +798,7 @@ mh_install (char *name, int automode)
 	}
       else
 	{
-	  mu_error(_("Cannot stat %s: %s"), name, mu_strerror (errno));
+	  mu_diag_funcall (MU_DIAG_ERROR, "stat", name, errno);
 	  exit (1);
 	}
     }
@@ -855,17 +870,17 @@ mh_set_reply_regex (const char *str)
 	      err ? err : "");
 }
 
-int
-mh_decode_2047 (char *text, char **decoded_text)
+const char *
+mh_charset (const char *dfl)
 {
-  const char *charset = mh_global_profile_get ("Charset", NULL);
+  const char *charset = mh_global_profile_get ("Charset", dfl);
 
   if (!charset)
-    return 1;
-  if (strcasecmp (charset, "auto") == 0)
+    return NULL;
+  if (mu_c_strcasecmp (charset, "auto") == 0)
     {
       /* Try to deduce the charset from LC_ALL variable */
-
+      
       char *lc_all = getenv ("LC_ALL");
       if (lc_all)
 	{
@@ -884,7 +899,13 @@ mh_decode_2047 (char *text, char **decoded_text)
 	  free (tmp);
 	}
     }
+  return charset;
+}
 
+int
+mh_decode_2047 (char *text, char **decoded_text)
+{
+  const char *charset = mh_charset (NULL);
   if (!charset)
     return 1;
   
@@ -929,17 +950,17 @@ mh_expand_aliases (mu_message_t msg,
 {
   mu_header_t hdr;
   size_t i, num;
-  char *buf;
+  const char *buf;
   
   mu_message_get_header (msg, &hdr);
   mu_header_get_field_count (hdr, &num);
   for (i = 1; i <= num; i++)
     {
-      if (mu_header_aget_field_name (hdr, i, &buf) == 0)
+      if (mu_header_sget_field_name (hdr, i, &buf) == 0)
 	{
-	  if (strcasecmp (buf, MU_HEADER_TO) == 0
-	      || strcasecmp (buf, MU_HEADER_CC) == 0
-	      || strcasecmp (buf, MU_HEADER_BCC) == 0)
+	  if (mu_c_strcasecmp (buf, MU_HEADER_TO) == 0
+	      || mu_c_strcasecmp (buf, MU_HEADER_CC) == 0
+	      || mu_c_strcasecmp (buf, MU_HEADER_BCC) == 0)
 	    {
 	      char *value;
 	      mu_address_t addr = NULL;
@@ -949,14 +970,13 @@ mh_expand_aliases (mu_message_t msg,
 	      
 	      mh_alias_expand (value, &addr, &incl);
 	      free (value);
-	      if (strcasecmp (buf, MU_HEADER_TO) == 0)
+	      if (mu_c_strcasecmp (buf, MU_HEADER_TO) == 0)
 		mu_address_union (addr_to, addr);
-	      else if (strcasecmp (buf, MU_HEADER_CC) == 0)
+	      else if (mu_c_strcasecmp (buf, MU_HEADER_CC) == 0)
 		mu_address_union (addr_cc ? addr_cc : addr_to, addr);
-	      else if (strcasecmp (buf, MU_HEADER_BCC) == 0)
+	      else if (mu_c_strcasecmp (buf, MU_HEADER_BCC) == 0)
 		mu_address_union (addr_bcc ? addr_bcc : addr_to, addr);
 	    }
-	  free (buf);
 	}
     }
 }
@@ -981,7 +1001,7 @@ mh_draft_message (const char *name, const char *msgspec, char **pname)
     {
       rc = mu_mailbox_uidnext (mbox, &uid);
       if (rc)
-	mu_error (_("Cannot obtain sequence number for the new message: %s"),
+	mu_error (_("cannot obtain sequence number for the new message: %s"),
 		  mu_strerror (rc));
     }
   else
@@ -993,7 +1013,7 @@ mh_draft_message (const char *name, const char *msgspec, char **pname)
       argv[1] = NULL;
       rc = mh_msgset_parse (mbox, &msgset, 1, argv, "cur");
       if (rc)
-	mu_error (_("Invalid message number: %s"), msgspec);
+	mu_error (_("invalid message number: %s"), msgspec);
       else if (msgset.count > 1)
 	mu_error (_("only one message at a time!"));
       else
