@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,70 +29,69 @@
 
 const char *program_version = "pick (" PACKAGE_STRING ")";
 static char doc[] = N_("GNU MH pick")"\v"
-N_("Options marked with `*' are not yet implemented.\n\
-Use -help to obtain the list of traditional MH options.");
+N_("Use -help to obtain the list of traditional MH options.");
 static char args_doc[] = N_("[messages]");
 
 /* GNU options */
 static struct argp_option options[] = {
   {"folder",  ARG_FOLDER, N_("FOLDER"), 0,
-   N_("Specify folder to operate upon"), 0},
+   N_("specify folder to operate upon"), 0},
 
   {N_("Specifying search patterns:"), 0,  NULL, OPTION_DOC,  NULL, 0},
   {"component", ARG_COMPONENT, N_("FIELD"), 0,
-   N_("Search the named header field"), 1},
+   N_("search the named header field"), 1},
   {"pattern", ARG_PATTERN, N_("STRING"), 0,
-   N_("A pattern to look for"), 1},
+   N_("set pattern to look for"), 1},
   {"search",  0, NULL, OPTION_ALIAS, NULL, 1},
   {"cflags",  ARG_CFLAGS,  N_("STRING"), 0,
-   N_("Flags controlling the type of regular expressions. STRING must consist of one or more of the following letters: B=basic, E=extended, I=ignore case, C=case sensitive. Default is \"EI\". The flags remain in effect until the next occurrence of --cflags option. The option must occur right before --pattern or --component option (or its alias).") },
+   N_("flags controlling the type of regular expressions. STRING must consist of one or more of the following letters: B=basic, E=extended, I=ignore case, C=case sensitive. Default is \"EI\". The flags remain in effect until the next occurrence of --cflags option. The option must occur right before --pattern or --component option (or its alias).") },
   {"cc",      ARG_CC,      N_("STRING"), 0,
-   N_("Same as --component cc --pattern STRING"), 1},
+   N_("same as --component cc --pattern STRING"), 1},
   {"date",    ARG_DATE,    N_("STRING"), 0,
-   N_("Same as --component date --pattern STRING"), 1},
+   N_("same as --component date --pattern STRING"), 1},
   {"from",    ARG_FROM,    N_("STRING"), 0,
-   N_("Same as --component from --pattern STRING"), 1},
+   N_("same as --component from --pattern STRING"), 1},
   {"subject", ARG_SUBJECT, N_("STRING"), 0,
-   N_("Same as --component subject --pattern STRING"), 1},
+   N_("same as --component subject --pattern STRING"), 1},
   {"to",      ARG_TO,      N_("STRING"), 0,
-   N_("Same as --component to --pattern STRING"), 1},
+   N_("same as --component to --pattern STRING"), 1},
 
   {N_("Date constraint operations:"), 0,  NULL, OPTION_DOC, NULL, 1},
   {"datefield",ARG_DATEFIELD, N_("STRING"), 0,
-   N_("Search in the named date header field (default is `Date:')"), 2},
+   N_("search in the named date header field (default is `Date:')"), 2},
   {"after",    ARG_AFTER,     N_("DATE"), 0,
-   N_("Match messages after the given date"), 2},
+   N_("match messages after the given date"), 2},
   {"before",   ARG_BEFORE,    N_("DATE"), 0,
-   N_("Match messages before the given date"), 2},
+   N_("match messages before the given date"), 2},
 
   {N_("Logical operations and grouping:"), 0, NULL, OPTION_DOC, NULL, 2},
   {"and",     ARG_AND,    NULL, 0,
-   N_("Logical AND (default)"), 3 },
+   N_("logical AND (default)"), 3 },
   {"or",      ARG_OR,     NULL, 0,
-   N_("Logical OR"), 3 },
+   N_("logical OR"), 3 },
   {"not",     ARG_NOT,    NULL, 0,
-   N_("Logical NOT"), 3},
+   N_("logical NOT"), 3},
   {"lbrace",  ARG_LBRACE, NULL, 0,
-   N_("Open group"), 3},
+   N_("open group"), 3},
   {"(",       0, NULL, OPTION_ALIAS, NULL, 3},
   {"rbrace",  ARG_RBRACE, NULL, 0,
-   N_("Close group"), 3},
+   N_("close group"), 3},
   {")",       0, NULL, OPTION_ALIAS, NULL, 3},
 
   {N_("Operations over the selected messages:"), 0, NULL, OPTION_DOC, NULL, 3},
   {"list",   ARG_LIST,       N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("List the numbers of the selected messages (default)"), 4},
+   N_("list the numbers of the selected messages (default)"), 4},
   {"nolist", ARG_NOLIST,     NULL, OPTION_HIDDEN, "", 4 },
   {"sequence", ARG_SEQUENCE,  N_("NAME"), 0,
-   N_("Add matching messages to the given sequence"), 4},
+   N_("add matching messages to the given sequence"), 4},
   {"public", ARG_PUBLIC, N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("Create public sequence"), 4},
+   N_("create public sequence"), 4},
   {"nopublic", ARG_NOPUBLIC, NULL, OPTION_HIDDEN, "", 4 },
   {"zero",     ARG_ZERO,     N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("Empty the sequence before adding messages"), 4},
+   N_("empty the sequence before adding messages"), 4},
   {"nozero", ARG_NOZERO, NULL, OPTION_HIDDEN, "", 4 },
   {"license", ARG_LICENSE, 0,      0,
-   N_("Display software license"), -1},
+   N_("display software license"), -1},
   {NULL},
 };
 
@@ -136,14 +136,14 @@ add_sequence (char *name)
 {
   if (!seq_list && mu_list_create (&seq_list))
     {
-      mu_error (_("Cannot create sequence list"));
+      mu_error (_("cannot create sequence list"));
       exit (1);
     }
   mu_list_append (seq_list, name);
 }
 
-static int
-opt_handler (int key, char *arg, void *unused, struct argp_state *state)
+static error_t
+opt_handler (int key, char *arg, struct argp_state *state)
 {
   char *s, *p;
   
@@ -263,7 +263,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       s = state->argv[state->next - 1];
       if (memcmp (s, "--", 2))
 	{
-	  argp_error (state, _("Invalid option -- %s"), s);
+	  argp_error (state, _("invalid option -- %s"), s);
 	  exit (1);
 	}
       p = strchr (s, '=');
@@ -276,7 +276,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
 	{
 	  if (state->next == state->argc)
 	    {
-	      mu_error (_("Invalid option -- %s"), s);
+	      mu_error (_("invalid option -- %s"), s);
 	      exit (1);
 	    }
 	  p = state->argv[state->next++];
@@ -290,7 +290,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
 
     default:
-      return 1;
+      return ARGP_ERR_UNKNOWN;
     }
 
   return 0;
@@ -347,8 +347,8 @@ main (int argc, char **argv)
   int flags;
 
   flags = mh_interactive_mode_p () ? 0 : ARGP_NO_ERRS;
-  mu_init_nls ();
-  mu_argp_init (program_version, NULL);
+  MU_APP_INIT_NLS ();
+  mh_argp_init (program_version);
   mh_argp_parse (&argc, &argv, flags, options, mh_option,
 		 args_doc, doc, opt_handler, NULL, &index);
   if (pick_parse (lexlist))
