@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2007, 2008, 2009, 2010 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -26,21 +27,21 @@ static char args_doc[] = "[file]";
 /* GNU options */
 static struct argp_option options[] = {
   {"alias",         ARG_ALIAS,         N_("FILE"), 0,
-   N_("Specify additional alias file") },
+   N_("specify additional alias file") },
   {"draft",         ARG_DRAFT,         NULL, 0,
-   N_("Use prepared draft") },
+   N_("use prepared draft") },
   {"draftfolder",   ARG_DRAFTFOLDER,   N_("FOLDER"), 0,
-   N_("Specify the folder for message drafts") },
+   N_("specify the folder for message drafts") },
   {"draftmessage",  ARG_DRAFTMESSAGE,  NULL, 0,
-   N_("Treat the arguments as a list of messages from the draftfolder") },
+   N_("treat the arguments as a list of messages from the draftfolder") },
   {"nodraftfolder", ARG_NODRAFTFOLDER, NULL, 0,
-   N_("Undo the effect of the last --draftfolder option") },
+   N_("undo the effect of the last --draftfolder option") },
   {"check",         ARG_CHECK,         N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("Check if addresses are deliverable") },
+   N_("check if addresses are deliverable") },
   {"nocheck",       ARG_NOCHECK,       NULL, OPTION_HIDDEN, "" },
 
   {"license",       ARG_LICENSE,       0,    0,
-   N_("Display software license"), -1},
+   N_("display software license"), -1},
 
   {NULL}
 };
@@ -58,10 +59,10 @@ struct mh_option mh_option[] = {
 
 static int check_recipients;
 static int use_draft;            /* Use the prepared draft */
-static char *draft_folder;       /* Use this draft folder */
+static const char *draft_folder; /* Use this draft folder */
 
-static int
-opt_handler (int key, char *arg, void *unused, struct argp_state *state)
+static error_t
+opt_handler (int key, char *arg, struct argp_state *state)
 {
   switch (key)
     {
@@ -100,7 +101,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
 
     default:
-      return 1;
+      return ARGP_ERR_UNKNOWN;
     }
   return 0;
 }
@@ -111,9 +112,9 @@ main (int argc, char **argv)
   int index;
   char *name = "draft";
   
-  mu_init_nls ();
+  MU_APP_INIT_NLS ();
   
-  mu_argp_init (program_version, NULL);
+  mh_argp_init (program_version);
   mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, &index);
 
