@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2007, 2008, 2009, 2010
+   Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -19,6 +20,7 @@
 #include "pop3d.h"
 #include <mailutils/argcv.h>
 #include <xalloc.h>
+#include "mailutils/libargp.h"
 
 int db_list (char *input_name, char *output_name);
 int db_make (char *input_name, char *output_name);
@@ -66,11 +68,11 @@ void (*argp_program_version_hook) (FILE *stream, struct argp_state *state) =
 static struct argp_option options[] = 
 {
   { NULL, 0, NULL, 0, N_("Actions are:"), 1 },
-  { "add", 'a', 0, 0, N_("Add user"), 1 },
-  { "modify", 'm', 0, 0, N_("Modify user's record (change password)"), 1 },
-  { "delete", 'd', 0, 0, N_("Delete user's record"), 1 },
-  { "list", 'l', 0, 0, N_("List the contents of DBM file"), 1 },
-  { "create", 'c', 0, 0, N_("Create the DBM from a plaintext file"), 1 },
+  { "add", 'a', 0, 0, N_("add user"), 1 },
+  { "modify", 'm', 0, 0, N_("modify user's record (change password)"), 1 },
+  { "delete", 'd', 0, 0, N_("delete user's record"), 1 },
+  { "list", 'l', 0, 0, N_("list the contents of DBM file"), 1 },
+  { "create", 'c', 0, 0, N_("create the DBM from a plaintext file"), 1 },
 
   { NULL, 0, NULL, 0,
     N_("Default action is:\n"
@@ -78,11 +80,11 @@ static struct argp_option options[] =
     "  For a user: --modify --username <username>\n"), 2 },
   
   { NULL, 0, NULL, 0, N_("Options are:"), 3 },
-  { "file", 'f', N_("FILE"), 0, N_("Read input from FILE (default stdin)"), 3 },
-  { "output", 'o', N_("FILE"), 0, N_("Direct output to file"), 3 },
-  { "password", 'p', N_("STRING"), 0, N_("Specify user's password"), 3 },
-  { "user", 'u', N_("USERNAME"), 0, N_("Specify user name"), 3 },
-  { "permissions", 'P', N_("PERM"), 0, N_("Force given permissions on the database"), 3 },
+  { "file", 'f', N_("FILE"), 0, N_("read input from FILE (default stdin)"), 3 },
+  { "output", 'o', N_("FILE"), 0, N_("direct output to file"), 3 },
+  { "password", 'p', N_("STRING"), 0, N_("specify user's password"), 3 },
+  { "user", 'u', N_("USERNAME"), 0, N_("specify user name"), 3 },
+  { "permissions", 'P', N_("PERM"), 0, N_("force given permissions on the database"), 3 },
   { NULL, }
 };
 
@@ -106,14 +108,14 @@ set_db_perms (struct argp_state *astate, char *opt, int *pperm)
 {
   int perm = 0;
    
-  if (isdigit(opt[0]))
+  if (mu_isdigit(opt[0]))
     {
       char *p;
       perm = strtoul (opt, &p, 8);
       if (*p)
 	{
-	  argp_error (astate, _("Invalid octal number: %s"), opt);
-	  exit (1);
+	  argp_error (astate, _("invalid octal number: %s"), opt);
+	  exit (EX_USAGE);
 	}
     }
   *pperm = perm;
@@ -198,11 +200,12 @@ main(int argc, char **argv)
   struct action_data adata;
 
   /* Native Language Support */
-  mu_init_nls ();
+  MU_APP_INIT_NLS ();
 
   mu_argp_init (program_version, NULL);
-  mu_argp_parse (&argp, &argc, &argv, 0,
-		 popauth_argp_capa, NULL, &adata);
+  if (mu_app_init (&argp, popauth_argp_capa, NULL,
+		   argc, argv, 0, NULL, &adata))
+    exit (EX_USAGE);
 
   return (*ftab[adata.action]) (&adata);
 }
@@ -213,7 +216,7 @@ check_action (int action)
   if (action != -1)
     {
       mu_error (_("You may not specify more than one `-aldp' option"));
-      exit (1);
+      exit (EX_USAGE);
     }
 }
 
@@ -234,17 +237,17 @@ check_user_perm (int action, struct action_data *ap)
 	  DBM_FILE db;
 	  if (mu_dbm_open (ap->input_name, &db, MU_STREAM_CREAT, permissions))
 	    {
-	      mu_error (_("Cannot create %s: %s"),
-			ap->input_name, mu_strerror (errno));
-	      exit (1);
+	      mu_diag_funcall (MU_DIAG_ERROR, "mu_dbm_open",
+			       ap->input_name, errno);
+	      exit (EX_SOFTWARE);
 	    }
 	  mu_dbm_close (db);
 	  mu_dbm_stat (ap->input_name, &sb);
 	}
       else
 	{
-	  mu_error (_("Cannot stat %s: %s"), ap->input_name, mu_strerror (errno));
-	  exit (1);
+	  mu_diag_funcall (MU_DIAG_ERROR, "stat", ap->input_name, errno);
+	  exit (EX_OSERR);
 	}
     }
 
@@ -255,17 +258,17 @@ check_user_perm (int action, struct action_data *ap)
   if (ap->username)
     {
       mu_error (_("Only the file owner can use --username"));
-      exit (1);
+      exit (EX_USAGE);
     }
 
   if (action != ACT_CHPASS)
     {
       mu_error (_("Operation not allowed"));
-      exit (1);
+      exit (EX_USAGE);
     }
   pw = getpwuid (uid);
   if (!pw)
-    exit (1);
+    exit (EX_OSERR);
   ap->username = pw->pw_name;
   return 1;
 }
@@ -279,9 +282,10 @@ action_list (struct action_data *ap)
   DBM_DATUM contents;
   
   check_user_perm (ACT_LIST, ap);
-  if (mu_dbm_open (ap->input_name, &db, MU_STREAM_READ, permissions))
+  if (mu_dbm_open (ap->input_name, &db, MU_STREAM_READ, 0))
     {
-      mu_error (_("Cannot open %s: %s"), ap->input_name, mu_strerror (errno));
+      mu_error (_("cannot open file %s: %s"),
+		ap->input_name, mu_strerror (errno));
       return 1;
     }
   
@@ -290,7 +294,7 @@ action_list (struct action_data *ap)
       fp = fopen (ap->output_name, "w");
       if (!fp)
 	{
-	  mu_error (_("Cannot create %s: %s"), ap->output_name, mu_strerror (errno));
+	  mu_error (_("cannot create file %s: %s"), ap->output_name, mu_strerror (errno));
 	  return 1;
 	}
     }
@@ -305,14 +309,17 @@ action_list (struct action_data *ap)
       MU_DATUM_SIZE (key) = strlen (ap->username);
       if (mu_dbm_fetch (db, key, &contents))
 	{
-	  mu_error (_("No such user: %s"), ap->username);
+	  mu_error (_("no such user: %s"), ap->username);
 	}
       else
-	fprintf (fp, "%.*s: %.*s\n",
-		 (int) MU_DATUM_SIZE (key),
-		 (char*) MU_DATUM_PTR (key),
-		 (int) MU_DATUM_SIZE (contents),
-		 (char*) MU_DATUM_PTR (contents));
+	{
+	  fprintf (fp, "%.*s: %.*s\n",
+		   (int) MU_DATUM_SIZE (key),
+		   (char*) MU_DATUM_PTR (key),
+		   (int) MU_DATUM_SIZE (contents),
+		   (char*) MU_DATUM_PTR (contents));
+	  mu_dbm_datum_free (&contents);
+	}
     }
   else
     {
@@ -326,6 +333,7 @@ action_list (struct action_data *ap)
 		   (char*) MU_DATUM_PTR (key),
 		   (int) MU_DATUM_SIZE (contents),
 		   (char*) MU_DATUM_PTR (contents));
+	  mu_dbm_datum_free (&contents);
 	}
     }
   
@@ -352,7 +360,8 @@ action_create (struct action_data *ap)
       fp = fopen (ap->input_name, "r");
       if (!fp)
 	{
-	  mu_error (_("Cannot open %s: %s"), ap->input_name, mu_strerror (errno));
+	  mu_error (_("cannot open file %s: %s"),
+		    ap->input_name, mu_strerror (errno));
 	  return 1;
 	}
     }
@@ -366,7 +375,7 @@ action_create (struct action_data *ap)
     ap->output_name = APOP_PASSFILE;
   if (mu_dbm_open (ap->output_name, &db, MU_STREAM_CREAT, permissions))
     {
-      mu_error (_("Cannot create %s: %s"), ap->output_name, mu_strerror (errno));
+      mu_error (_("cannot create database %s: %s"), ap->output_name, mu_strerror (errno));
       return 1;
     }
 
@@ -426,7 +435,8 @@ open_io (int action, struct action_data *ap, DBM_FILE *db, int *not_owner)
     *not_owner = rc;
   if (mu_dbm_open (ap->input_name, db, MU_STREAM_RDWR, permissions))
     {
-      mu_error (_("Cannot open %s: %s"), ap->input_name, mu_strerror (errno));
+      mu_error (_("cannot open file %s: %s"),
+		ap->input_name, mu_strerror (errno));
       return 1;
     }
   return 0;
@@ -444,7 +454,7 @@ fill_pass (struct action_data *ap)
 	  free (ap->passwd);
 	p = getpass (_("Password:"));
 	if (!p)
-	  exit (1);
+	  exit (EX_DATAERR);
 	ap->passwd = strdup (p);
 	/* TRANSLATORS: Please try to format this string so that it has
 	   the same length as the translation of 'Password:' above */
@@ -466,7 +476,7 @@ action_add (struct action_data *ap)
   
   if (!ap->username)
     {
-      mu_error (_("Missing username to add"));
+      mu_error (_("missing username to add"));
       return 1;
     }
 
@@ -484,7 +494,7 @@ action_add (struct action_data *ap)
 
   rc = mu_dbm_insert (db, key, contents, 1);
   if (rc)
-    mu_error (_("Cannot store datum"));
+    mu_error (_("cannot store datum"));
 
   mu_dbm_close (db);
   return rc;
@@ -499,7 +509,7 @@ action_delete (struct action_data *ap)
   
   if (!ap->username)
     {
-      mu_error (_("Missing username to delete"));
+      mu_error (_("missing username to delete"));
       return 1;
     }
 
@@ -511,7 +521,7 @@ action_delete (struct action_data *ap)
 
   rc = mu_dbm_delete (db, key);
   if (rc)
-    mu_error (_("Cannot remove record for %s"), ap->username);
+    mu_error (_("cannot remove record for %s"), ap->username);
 
   mu_dbm_close (db);
   return rc;
@@ -531,7 +541,7 @@ action_chpass (struct action_data *ap)
 
   if (!ap->username)
     {
-      mu_error (_("Missing username"));
+      mu_error (_("missing username"));
       return 1;
     }
 
@@ -542,7 +552,7 @@ action_chpass (struct action_data *ap)
   MU_DATUM_SIZE (key) = strlen (ap->username);
   if (mu_dbm_fetch (db, key, &contents))
     {
-      mu_error (_("No such user: %s"), ap->username);
+      mu_error (_("no such user: %s"), ap->username);
       return 1;
     }
 
@@ -565,11 +575,12 @@ action_chpass (struct action_data *ap)
 
   fill_pass (ap);
   
+  mu_dbm_datum_free (&contents);
   MU_DATUM_PTR (contents) = ap->passwd;
   MU_DATUM_SIZE (contents) = strlen (ap->passwd);
   rc = mu_dbm_insert (db, key, contents, 1);
   if (rc)
-    mu_error (_("Cannot replace datum"));
+    mu_error (_("cannot replace datum"));
 
   mu_dbm_close (db);
   return rc;
@@ -583,12 +594,14 @@ popauth_version (FILE *stream, struct argp_state *state)
 #elif defined(WITH_BDB)
 # define FORMAT "Berkeley DB"
 #elif defined(WITH_NDBM)
-# define FORMAT "NDBM"  
+# define FORMAT "NDBM"
 #elif defined(WITH_OLD_DBM)
-# define FORMAT "Old DBM"  
+# define FORMAT "Old DBM"
+#elif defined(WITH_TOKYOCABINET)
+# define FORMAT "Tokyo Cabinet"
 #endif
   printf ("%s\n", argp_program_version);
   printf (_("Database format: %s\n"), FORMAT);
   printf (_("Database location: %s\n"), APOP_PASSFILE);
-  exit (EXIT_SUCCESS);
+  exit (EX_OK);
 }
