@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005, 2007, 2009, 2010 Free Software Foundation,
+   Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -23,16 +24,16 @@
 #ifdef HAVE_PGSQL
 #include <mailutils/mailutils.h>
 #include <mailutils/sql.h>
+#include <mailutils/cctype.h>
 
 #include <libpq-fe.h>
-#include <ctype.h>
 
 static char *
 chop (char *str)
 {
   int len;
   
-  for (len = strlen(str); len > 0 && isspace(str[len-1]); len--)
+  for (len = strlen(str); len > 0 && mu_isspace(str[len-1]); len--)
     ;
   str[len] = 0;
   return str;
@@ -46,7 +47,7 @@ struct mu_pgsql_data
 
 
 static int
-init (mu_sql_connection_t conn)
+postgres_init (mu_sql_connection_t conn)
 {
   struct mu_pgsql_data *dp = calloc (1, sizeof (*dp));
   if (!dp)
@@ -56,7 +57,7 @@ init (mu_sql_connection_t conn)
 }
 
 static int
-destroy (mu_sql_connection_t conn)
+postgres_destroy (mu_sql_connection_t conn)
 {
   struct mu_pgsql_data *dp = conn->data;
   free (dp);
@@ -65,7 +66,7 @@ destroy (mu_sql_connection_t conn)
 }
 
 static int
-connect (mu_sql_connection_t conn)
+postgres_connect (mu_sql_connection_t conn)
 {
   struct mu_pgsql_data *dp = conn->data;
   char portbuf[16];
@@ -88,7 +89,7 @@ connect (mu_sql_connection_t conn)
 }
 
 static int 
-disconnect (mu_sql_connection_t conn)
+postgres_disconnect (mu_sql_connection_t conn)
 {
   struct mu_pgsql_data *dp = conn->data;
   PQfinish (dp->pgconn);
@@ -96,7 +97,7 @@ disconnect (mu_sql_connection_t conn)
 }
 
 static int
-query (mu_sql_connection_t conn, char *query)
+postgres_query (mu_sql_connection_t conn, char *query)
 {
   struct mu_pgsql_data *dp = conn->data;
   ExecStatusType stat;
@@ -114,13 +115,13 @@ query (mu_sql_connection_t conn, char *query)
 }
 
 static int
-store_result (mu_sql_connection_t conn)
+postgres_store_result (mu_sql_connection_t conn)
 {
   return 0;
 }
 
 static int
-release_result (mu_sql_connection_t conn)
+postgres_release_result (mu_sql_connection_t conn)
 {
   struct mu_pgsql_data *dp = conn->data;
   PQclear (dp->res);
@@ -129,7 +130,7 @@ release_result (mu_sql_connection_t conn)
 }
 
 static int
-num_columns (mu_sql_connection_t conn, size_t *np)
+postgres_num_columns (mu_sql_connection_t conn, size_t *np)
 {
   struct mu_pgsql_data *dp = conn->data;
   if (!dp->res)
@@ -139,7 +140,7 @@ num_columns (mu_sql_connection_t conn, size_t *np)
 }
 
 static int
-num_tuples (mu_sql_connection_t conn, size_t *np)
+postgres_num_tuples (mu_sql_connection_t conn, size_t *np)
 {
   struct mu_pgsql_data *dp = conn->data;
   if (!dp->res)
@@ -149,7 +150,8 @@ num_tuples (mu_sql_connection_t conn, size_t *np)
 }
 
 static int
-get_column (mu_sql_connection_t conn, size_t nrow, size_t ncol, char **pdata)
+postgres_get_column (mu_sql_connection_t conn, size_t nrow, size_t ncol,
+		     char **pdata)
 {
   struct mu_pgsql_data *dp = conn->data;
   if (!dp->res)
@@ -159,7 +161,8 @@ get_column (mu_sql_connection_t conn, size_t nrow, size_t ncol, char **pdata)
 }
 
 static int
-get_field_number (mu_sql_connection_t conn, const char *fname, size_t *fno)
+postgres_get_field_number (mu_sql_connection_t conn, const char *fname,
+			   size_t *fno)
 {
   struct mu_pgsql_data *dp = conn->data;
   if (!dp->res)
@@ -170,7 +173,7 @@ get_field_number (mu_sql_connection_t conn, const char *fname, size_t *fno)
 }
 
 static const char *
-errstr (mu_sql_connection_t conn)
+postgres_errstr (mu_sql_connection_t conn)
 {
   struct mu_pgsql_data *dp = conn->data;
   return PQerrorMessage (dp->pgconn);
@@ -180,18 +183,18 @@ errstr (mu_sql_connection_t conn)
 MU_DECL_SQL_DISPATCH_T(postgres) = {
   "postgres",
   5432,
-  init,
-  destroy,
-  connect,
-  disconnect,
-  query,
-  store_result,
-  release_result,
-  num_tuples,
-  num_columns,
-  get_column,
-  get_field_number,
-  errstr,
+  postgres_init,
+  postgres_destroy,
+  postgres_connect,
+  postgres_disconnect,
+  postgres_query,
+  postgres_store_result,
+  postgres_release_result,
+  postgres_num_tuples,
+  postgres_num_columns,
+  postgres_get_column,
+  postgres_get_field_number,
+  postgres_errstr,
 };
 
 #endif
